<?php

/** 
 * Class Filter
 * Adds the Filter to the Project. Based on wp-json Rest, filtering by distance, place, termns
 * @package    LeadGenerator
 * @subpackage Filter
 * @since      1.0.0
 *
 */
defined('ABSPATH') || exit;
class Filter
{
    private $distance_fields;
    private $query;
    private $date_times = [];
    /**
     * Constructor for the class.
     * Initializes the distance fields and date times arrays.
     * Adds shortcodes for rendering the filter, sorting, counting, selected filters, search form, and pagination.
     * Adds actions for loading scripts and handling AJAX callbacks.
     */
    /**
     * Constructor for the Leadgen class.
     * Initializes the namespace and registers REST routes for job listings and address search.
     *
     * @since 1.0.0
     */
    function __construct()
    {
        $this->distance_fields = [
            5,
            10,
            25,
            50,
            75,
            100
        ];

        $this->date_times = [
            'today' => 'Heute',
            '-3' => 'letzten 3 Tage',
            '-7' => 'letzten 7 Tage',
            '-14' => 'letzten 14 Tage',
            '-30' => 'letzten 30 Tage',
            '-9000' => 'Alle',
        ];

        add_shortcode('filter_jobs', [$this, 'render_complete_filter']);
        add_shortcode('filter_sort', [$this, 'show_sorting']);
        add_shortcode('filter_count', [$this, 'show_counting']);
        add_shortcode('filter_selected', [$this, 'filter_selected']);
        add_shortcode('search_form', [$this, 'search_form_shortcode']);
        add_shortcode('filter_pagination', [$this, 'render_pagination']);
        add_action('wp_enqueue_scripts', [$this, 'load_scripts']);


    }
    /**
     * Builds the selected options for a given set of arguments.
     *
     * @param array $args The arguments to be used in building the selected options.
     * @return array The selected options.
     */
    private static function build_selected_options($args)
    {
        $date_selected = '';
        if (isset($args['date_query'])) {
            if (isset($args['date_query']['after'])) {

                switch ($args['date_query']['after']) {
                    case "30 days ago":
                        $date_selected = "letzten 30 Tage";
                        break;
                    case "14 days ago":
                        $date_selected = "letzten 14 Tage ";
                        break;
                    case "7 days ago":
                        $date_selected = "letzten 7 Tage";
                        break;
                    case "3 days ago":
                        $date_selected = "letzten 3 Tage";
                        break;
                    case "today":
                        $date_selected = "Heute";
                        break;
                }
            }

        }

        // $tax_queries = 
        array_shift($args['tax_query']);

        if (isset($args['tax_query'])) {
            // var_dump($args['tax_query']);
            $term_ids = [];
            foreach ($args['tax_query'] as $tax_query) {
                // var_dump($tax_query);
                if (gettype($tax_query['terms']) == 'array') {
                    foreach ($tax_query['terms'] as $term_id) {
                        $term_ids[$term_id] = get_term($term_id)->name;

                    }
                } elseif (gettype($tax_query['terms']) == 'string') {
                    $term_ids[$tax_query['terms']] = get_term($tax_query['terms'])->name;

                }

            }
            if (isset($args['date_query']['after'])) {
                $val = Filter::get_reverse_date_query($args['date_query']['after']);
                if (isset($date_selected) && $date_selected != '') {
                    $term_ids[$val] = $date_selected;
                }
            }



            $term_ids = array_filter($term_ids);

            return Filter::selected($term_ids);
        }

    }
    /**
     * Renders a selected filter template using Timber.
     *
     * @param array $options An array of options to be displayed in the filter.
     * @return string The rendered template.
     */
    private static function selected($options)
    {

        //  $template = wp_leadgen_template_path'';
        $template = apply_filters('wp_leadgen_template_path', 'filter/selected.twig');
        $data = [
            'options' => $options,
        ];
        return Timber::compile($template, $data);


    }
    /**
     * Returns a string containing the selected filter options for the current query.
     *
     * @param array $selected An array of selected filter options.
     * @return string A string containing the selected filter options for the current query.
     */
    public function filter_selected($selected = [])
    {
        global $wp_query;
        $term_id = $wp_query->queried_object_id;
        $cat_name = $wp_query->queried_object->name;

        $selected[$term_id] = $cat_name;
        if (isset($wp_query->found_posts)) {
            $count = $wp_query->found_posts;
        } else {
            $count = 0;
        }
        $template = '<div id="filter_selected_bar">';
        $template .= '<span> <span id="filter_count">' . $count . '</span> Jobs  <span id="filter_selected_bar_selections">';
        if ($cat_name == 'job') {
            $template .= ' in ';
        } else {
            $template .= ' <span id="filter_select">' . $this->selected($selected) . '</span>';
        }
        $template .= '</span>';
        $template .= '</span>';
        $template .= '</div>';
        return $template;


    }
    /**
     * Loads necessary scripts for the plugin.
     *
     * @return void
     */
    public function load_scripts()
    {

        global $wp_query;
        $file_path = LGenURL . 'dist/main.js';
        $file_size = filesize(LGenPath . '/dist/main.js');
        wp_register_script('jobs', $file_path, ["google-maps"], $file_size);
        wp_enqueue_script('jobs');
        $activeTerm = '';
        $activeTermTaxonomy = '';
        if ($wp_query->queried_object) {
            if (isset($wp_query->queried_object->term_id)) {

                $activeTerm = $wp_query->queried_object->term_id;
                $activeTermTaxonomy = $wp_query->queried_object->taxonomy;
            }
        } else {
            $activeTerm = '';
        }
        wp_localize_script(
            'jobs',
            'dp_args',
            [
                'restUrl' => rest_url(),
                'restNonce' => wp_create_nonce('wp_ajax_nonce'),
                'activeTerm' => $activeTerm,
                'activeTermTaxonomy' => $activeTermTaxonomy,
                'jobArchive' => get_post_type_archive_link('job'),

            ]
        );

    }
    /**
     * Retrieves all premium jobs and renders them in a job wrapper div.
     *
     * @param array $args Optional arguments for filtering the premium jobs.
     * @return string HTML output of the premium job items.
     */
    static function get_premium_results($args)
    {


        $jobQuery = new LGenQuery();
        $jobs = $jobQuery->get_all_premium_jobs($args);
        $LeadPost = new LGenPost();

        $output = '<div class="job-wrapper premium">';
        foreach ($jobs as $job) {

            $output .= $LeadPost->render_job_item($job);
        }
        $output .= '</div>';
        return $output;
    }
    /**
     * Retrieves all job results based on the given arguments.
     *
     * @param array $args Optional. Arguments to filter the job results by.
     * @return string HTML output of the job results.
     */
    static function get_job_results($args)
    {



        $jobQuery = new LGenQuery();
        $jobs = $jobQuery->get_all_jobs($args);

        $LeadPost = new LGenPost();

        $output = '<div class="job-wrapper">';
        if (count($jobs) > 0) {
            foreach ($jobs as $job) {
                $output .= $LeadPost->render_job_item($job);
            }
        } else {
            $output .= $LeadPost->render_job_404();
        }

        $output .= '</div>';
        return $output;
    }
    /**
     * Renders the pagination for the current query.
     *
     * @return string The pagination HTML.
     */
    public function render_pagination()
    {
        global $wp_query;


        $posts_per_page = $wp_query->query_vars['posts_per_page'];
        $paged = $wp_query->query_vars['paged'];
        $found_posts = $wp_query->found_posts;
        $max_num_pages = $wp_query->max_num_pages;

        return self::get_pagination($posts_per_page, $paged, $found_posts, $max_num_pages, False);
    }
    /**
     * Retrieves the pagination for a given set of posts.
     *
     * @param int $posts_per_page The number of posts to display per page.
     * @param int $paged The current page number.
     * @param int $found_posts The total number of posts found.
     * @param int $max_num_pages The maximum number of pages to display.
     * @return string The compiled pagination template.
     */
    static function get_pagination($posts_per_page, $paged, $found_posts, $max_num_pages, $form_id = False)
    {
    
        if ($max_num_pages < 1) {
            return;
        }


        $template = apply_filters('wp_leadgen_template_path', '/filter/pagination.twig');

        $data = [
            'paged' => $paged,
            'found_posts ' => $found_posts,
            'posts_per_page ' => $posts_per_page,
            'max_num_pages' => $max_num_pages,
            'form_id' => $form_id
        ];


        return Timber::compile($template, $data);


    }
    /**
     * Retrieves job results based on given arguments.
     *
     * @param array $args Arguments to filter job results.
     * @return array Array containing job results and related information.
     */
    static function get_results($args)
    {

        $jobQuery = new LGenQuery();

        $jobs = $jobQuery->get_all_jobs($args);
        $count = $jobQuery->found_posts($args);

        $LeadPost = new LGenPost();
        $json_output = [];



        $json_output['page'] = $count;
        $json_output['total_pages'] = $count;
        $json_output['options'] = Filter::build_selected_options($args);


        $json_output['found_posts'] = $count;
        $json_output['jobs']['output'] = '';
        $json_output['jobs']['count'] = count($jobs);
        $output = '';
        foreach ($jobs as $job) {
            $output .= $LeadPost->render_job_item($job);
        }
        $json_output['jobs']['output'] = $output;
        $premium_jobs = $jobQuery->get_all_premium_jobs($args);
        $json_output['premium']['output'] = '';
        $json_output['premium']['count'] = count($premium_jobs);
        foreach ($premium_jobs as $job) {

            $output .= $LeadPost->render_job_item($job);
        }
        $json_output['premium']['output'] = $output;

        $json_output['query'] = $jobQuery->get_query($args);
        return $json_output;
    }
    /**
     * Displays the filter form with the given arguments.
     *
     * @param array $args Optional. Additional arguments for the filter form. Default empty array.
     * @return string Rendered HTML of the filter form.
     */
    public function show_filter($args = [], $atts = [])
    {
        $s_placeholder = 'Suche';
        $city = 'Berlin';
        $city_placeholder = '';
        $radius = '';
        $pre = False;
        if (isset($atts['s_placeholder'])) {
            $s_placeholder = $atts['s_placeholder'];
        }
        if (isset($atts['s'])) {
            $s = $atts['s'];
            $pre = True;
        }
        if (isset($atts['city'])) {
            $city = $atts['city'];
            $pre = True;
        }
        if (isset($atts['radius'])) {
            $radius = $atts['radius'];
            $pre = True;
        }

        if (isset($atts['city_placeholder'])) {
            $city_placeholder = $atts['city_placeholder'];

        }


        $data = [
            'pre' => $pre,
            'radius' => $radius,
            'city' => $city,
        //    's' => $s,
            's_placeholder' => $s_placeholder,
            'city_placeholder' => $city_placeholder,
            'job_district' => $this->show_filter_term('job_district'),
            'distances' => $this->show_distance_filter(),
            'sorting' => $this->show_sorting(),
            'atts' => $atts
        ];
        if (isset($s)){
            $data['s'] = $s;
        }

        if (isset($atts['terms'])) {
            $data['terms'] = $this->get_selected_terms($atts['terms'], $atts['bys'], $atts['values']);
        }


        $template = apply_filters('wp_leadgen_template_path', 'forms.twig');

        $rendered_html = Timber::compile($template, $data);
        return $rendered_html;
    }
    /**
     * Removes a specific element from the sidebar.
     *
     * @param array $args The arguments used to specify the element to be removed.
     * @param array $data The data containing the sidebar elements.
     * @return array The updated data with the specified element removed.
     */
    private function disable_sidebar_el($args, $data)
    {

        $data[key($args)] = [];
        unset($data[key($args)]);

        return $data;
    }
    /**
     * Retrieves the number of posts by a given date.
     *
     * @param array $args Additional arguments for the post query.
     * @param string $date Date to query posts by.
     *
     * @return int Number of posts matching the given date.
     */
    private function get_post_count_by_date($args, $date)
    {
        $jobQuery = new LGenQuery();

        $date_query = [
            "posts_per_page" => -1
        ];

        $date_query['date_query'] = $this->get_date_query($date);

        $args = array_merge($jobQuery->args, $args, $date_query);

        return count(get_posts($args));
    }

    private function get_main_cat_from_args($args)
    {
        if ($args['tax_query'] && isset($args['tax_query'])) {
            if (isset($args['tax_query'][0])) {
                if ($args['tax_query'][0]['terms']) {
                    $term = get_term_by('slug', $args['tax_query'][0]['terms'], $args['tax_query'][0]['taxonomy']);
                    return $term;
                }
            }

        }
    }
    /**
     * Builds an array of data for each date and time, including the key, label, and post count.
     *
     * @param array $args The arguments to be used in the get_post_count_by_date method.
     * @return array An array of date and time data, including the key, label, and post count.
     */
    private function build_datetime_data($args)
    {

        $date_post_array = [];
        foreach ($this->date_times as $k => $date) {
            $count = $this->get_post_count_by_date($args, $k);
            $date_post = [
                'key' => $k,
                'label' => $date,
                'count' => $count
            ];
            $date_post_array[] = $date_post;


        }
        return $date_post_array;
    }
    /**
     * Displays the sidebar with filtered job data.
     *
     * @param array $args Optional. Arguments to filter the job data.
     * @return string Rendered HTML of the sidebar.
     */
    public function show_sidebar($args = [])
    {


        $jobQuery = new LGenQuery();


        $the_query = new WP_Query($jobQuery->build_args($args));
        $total_posts = $the_query->found_posts;


        global $wp_query;
        $global_term_id = '';
        $global_term = False;
        if (isset($wp_query->queried_object) && isset($wp_query->queried_object->term_id)) {
            $global_term_id = $wp_query->queried_object->term_id;
            $global_term = get_term_by('term_id', $global_term_id, 'jobcategory');
        }


   


        if (
            isset($args['job_working_time_str'])
            || isset($args['job_form_of_employment_str'])
            || isset($args['jobcategory_str'])
            || isset($args['job_type_str'])
            || isset($args['job_district_str'])
        ) {
            // Handle taxonomies
            $taxonomies = ['jobcategory', 'job_form_of_employment', 'job_working_time', 'working_time', 'job_type'];


            foreach ($taxonomies as $taxonomy) {
                $taxonomy_str = $taxonomy . '_str';

                if (!empty($args[$taxonomy_str])) {

                    $args['tax_query'][] = [
                        'taxonomy' => $taxonomy,
                        'field' => 'name',
                        'terms' => $args[$taxonomy_str],

                    ];
                }


            }



            if (isset($args['tax_query'])) {

                $global_term = $this->get_main_cat_from_args($args);
                $args['main_cat'] = $global_term;


                if (isset($args['tax_query'][0])) {
                    $global_term = get_term_by('slug', $args['tax_query'][0]['terms'], $args['tax_query'][0]['taxonomy']);
                }

            }

        }




        // Get ID of the global term
        $date_terms = $this->build_datetime_data($args);

        $args['tax_query']['relation'] = 'OR';


        $data = [
            'date_times' => $date_terms,
            'job_type' => $this->show_filter_term('job_type', $args, $global_term),
            'jobcategory' => $this->show_filter_term('jobcategory', $args, $global_term),
            'job_form_of_employment' => $this->show_filter_term('job_form_of_employment', $args, $global_term),
            'job_working_time' => $this->show_filter_term('job_working_time', $args, $global_term),
            'count' => $total_posts,
            'args' => $args
        ];
        $data = $this->disable_sidebar_el($args, $data);
        $rendered_html = Timber::compile('sidebar.twig', $data);
        return $rendered_html;
    }


    public function get_filter_bar($args = [])
    {


        $term = get_term_by($args['by'], $args['value'], $args['term']);

        $data = [
            'term' => $term,
            'args' => $args
        ];
        global $active_terms;
        $active_terms[$args['term']] = $term->term_id;

        $rendered_html = Timber::compile('filter/bar/term-checkboxes.twig', $data);
        return $rendered_html;
    }

    /**
     * Retrieves and displays filter terms for a given taxonomy.
     *
     * @param string $name The name of the taxonomy to retrieve terms from.
     * @param array $args Optional. Additional arguments for retrieving terms. Default empty array.
     * @param int|bool $global_term_id Optional. The global term ID to use for counting posts. Default false.
     * @return array An array of terms for the given taxonomy.
     */
    public function show_filter_term($name, $args = [], $global_term = False)
    {



        $terms = $this->get_categories_hierarchical(

            array(
                'taxonomy' => $name,
                'hide_empty' => true,
            )
        );

        foreach ($terms as $term) {
            if (isset($global_term) && ($global_term) && is_int($global_term->term_id)) {
                if ($term->taxonomy != $global_term->taxonomy) {
                    $count = Filter::count_cum_posts_in_term($term->term_id, $name, $global_term, $args);


                } else {
                    $count = Filter::count_posts_in_term($term->term_id, $name, $args);
                }
                $term->post_count = $count;
                $term->count = $count;

            } else {

                $count = Filter::count_posts_in_term($term->term_id, $name, $args);
                $term->post_count = $count;
                $term->count = $count;


            }


        }

        return $terms;
    }
    /**
     * Counts the number of posts in a given term.
     *
     * @param array $term_ids Array of term IDs to count posts in.
     * @param string $name Name of the taxonomy to use for the query.
     * @param array $args Optional. Additional arguments for the query.
     * @return int Number of posts in the given term.
     */
    public static function count_posts_in_term($term_ids, $name, $args = [])
    {


        $query_args = array(
            'posts_per_page' => -1, // retrieve all posts
            'post_type' => 'job',
            'post_status' => 'publish',
            'fields' => 'ids', // only return post IDs to speed up the query
            'tax_query' => array(
                'operator' => 'AND',
                array(
                    'taxonomy' => $name,
                    'field' => 'term_id',
                    'terms' => $term_ids,
                    'operator' => 'IN',
                ),


            ),
        );
        $args["tax_query"] = $query_args['tax_query'];

        $merged_args = array_merge($args, $query_args);
        if ($name == 'job_type') {
            //  exit();
        }
        $query = new WP_Query($merged_args);


        return $query->post_count;
    }
    /**
     * Counts the total number of posts that have a specific term assigned to them.
     *
     * @param array $ids The IDs of the terms to count posts for.
     * @param string $name The name of the taxonomy to search within.
     * @param array $args Optional. Additional arguments to pass to the WP_Query object.
     * @return int The total number of posts that have the specified term assigned to them.
     */
    public static function count_cum_posts_in_term($ids, $name, $global_term, $args = [])
    {

        if (isset($args['post__in'])) {

            $query_args['post__in'] = $args['post__in'];

        }


        $ids = array_filter([$ids]);
        $query_args = array(
            'posts_per_page' => -1, // retrieve all posts
            'fields' => 'ids', // only return post IDs to speed up the query
            'tax_query' => array(
                'operator' => 'AND',
                [
                    "taxonomy" => $name,
                    "field" => "term_id",
                    "terms" => $ids,
                    "operator" => "IN",
                ],
                [
                    "taxonomy" => $global_term->taxonomy,
                    "field" => "term_id",
                    "terms" => $global_term->term_id,
                    "operator" => "IN",
                ]



            ),
        );
        $args["tax_query"] = $query_args['tax_query'];

        //       $merged_args = array_merge($args, $query_args);
        //var_dump($merged_args);
        $query = new WP_Query($args);


        return $query->post_count;
    }
    /**
     * Retrieves autocomplete suggestions for cities using an external API.
     * 
     * @param WP_REST_Request $request The request object.
     * @return array An array of autocomplete suggestions or an error message if no suggestions are found.
     */
    public function call_autocomplete_cities(WP_REST_Request $request)
    {
        $term = $request->get_param('term');
        /* @ToDo add Adminform to allow custom api */
        $response = wp_remote_get('https://api.jsonbix.com/places/v1/autocomplete?q=' . $term);
        if (is_wp_error($response)) {
            return array('error' => 'Unable to communicate with the external API');
        }

        $body = wp_remote_retrieve_body($response);

        $data = json_decode($body);

        if (!empty($data) && is_array($data)) {
            return $data;
        } else {
            return array('error' => 'Kein Ort gefunden');
        }
    }


    /**
     * Retrieves hierarchical categories based on given arguments.
     *
     * @param array $args Optional. Array of arguments for retrieving categories. Default empty array.
     * @return array Array of categories.
     */
    /**
     * Retrieves hierarchical categories based on given arguments.
     *
     * @param array $args Optional. Array of arguments for retrieving categories.
     * @return array Array of categories.
     */
    public function get_categories_hierarchical($args = array(), $taxonomy = '', $term_name = '')
    {
        if (!isset($args['parent']))
            $args['parent'] = 0;

        $categories = get_terms($args);

        foreach ($categories as $key => $category):

            if ($taxonomy && $term_name) {


                //Creating a new query for each category to get the count of custom posts
                $post_args = array(
                    'post_type' => 'job',
                    'tax_query' => array(
                        'operator' => 'AND',
                        array(
                            'taxonomy' => $taxonomy,
                            'field' => 'name',
                            'terms' => $category->term_id,
                        ),
                        array(
                            'taxonomy' => $taxonomy,
                            'field' => 'name',
                            'terms' => $category->term_id,
                        ),
                    ),
                );

                $the_query = new WP_Query($post_args);

                //Replacing the default count with the count from the query
                $categories[$key]->count = $the_query->found_posts;
            }
            // $categories[$key]->child_categories = $this->get_categories_hierarchical($args, $taxonomy, $term_name);

        endforeach;

        return $categories;
    }
    /**
     * Retrieves distinct meta values for a given field from the postmeta table for posts of type 'job'.
     *
     * @param string $field The name of the meta field to retrieve values for.
     * @return array An array of distinct meta values for the given field.
     */
    public function show_field_filter($field)
    {
        global $wpdb;

        $values = $wpdb->get_results("SELECT DISTINCT meta_value as name FROM $wpdb->postmeta AS pm, $wpdb->posts AS p WHERE
        meta_key = '" . $field . "' and pm.post_id = p.ID and p.post_type = 'job'", ARRAY_A);
        return $values;



    }
    /**
     * Returns the distance fields for filtering.
     *
     * @return array The distance fields for filtering.
     */
    public function show_distance_filter()
    {
        return $this->distance_fields;

    }
    /**
     * Renders a counting template with the given options.
     *
     * @param int $count The number of options to be displayed.
     * @return string The rendered counting template.
     */
    public function show_counting($count)
    {


        $template = '/filter/counting.twig';
        $data = [
            'options' => $count,
        ];


        return Timber::compile($template, $data);

    }
    /**
     * Displays a sorting filter for a list of items.
     *
     * @param array $atts Optional. An array of attributes for the sorting filter. Default empty.
     * @return string The rendered HTML for the sorting filter.
     */
    public function show_sorting($atts = [])
    {
        $options = shortcode_atts(
            [
                'asc' => 'asc',
                'desc' => 'desc',
                'id' => 'term',
            ],
            $atts,
        );

        $sort_options = [
            'rand' => 'Standardsortierung',
            'date' => 'Aktuelle zuerst',
            'distance' => 'Entfernung',
        ];

        $template = '/filter/sortings.twig';
        $data = [
            'sort_options' => $sort_options,
            'options' => $options,
        ];


        return Timber::compile($template, $data);

    }
    /**
     * Returns the HTML output for the filter submit button.
     *
     * @return string The HTML output for the filter submit button.
     */
    public function show_filter_submit()
    {

        $output = '<button type="submit" value="Search" id="dp-submit" class="" aria-label="' . __('Search', 'tea') . '"> <svg
                width="24" height="24" viewBox="0 0 24 24" fill="none" xmlns="http://www.w3.org/2000/svg">
                <path d="M4 12H20" stroke="url(#paint0_linear_216_90)" stroke-width="2" stroke-linecap="round"
                    stroke-linejoin="round" />
                <path d="M13.3333 5.33325L19.9997 11.9999L13.3331 18.6666" stroke="url(#paint1_linear_216_90)" stroke-width="2"
                    stroke-linecap="round" stroke-linejoin="round" />
                <defs>
                    <linearGradient id="paint0_linear_216_90" x1="4" y1="12.5" x2="20" y2="12.5" gradientUnits="userSpaceOnUse">
                        <stop stop-color="#FD3F77" />
                        <stop offset="1" stop-color="#FF7E47" />
                    </linearGradient>
                    <linearGradient id="paint1_linear_216_90" x1="13.333" y1="11.9999" x2="19.9997" y2="11.9999"
                        gradientUnits="userSpaceOnUse">
                        <stop stop-color="#FD3F77" />
                        <stop offset="1" stop-color="#FF7E47" />
                    </linearGradient>
                </defs>
            </svg>
        </button>';

        return $output;

    }
    /**
     * Calls the OpenStreetMap API to geocode a city or search term.
     *
     * @param array $request The request parameters.
     * @return array The geocoded data.
     */
    public function call_openstreet_map($request)
    {
        $params = $request->get_params($request);
        if (isset($params['city'])) {

            return Nominatim::geocode_city($params['city']);
        }
        if (isset($params['search'])) {
            return Nominatim::geocode($params['search']);
        }
    }
    /**
     * Builds arguments for a query.
     *
     * @param array $args The arguments to be built.
     * @return array The built arguments.
     */
    public function build_args($args)
    {

        $args['post_type'] = 'job';
        // Initialize tax_query with a default relation
        $tax_query = ['relation' => 'OR'];

        // Filter posts
        if (isset($args["post__not_in"])) {
            $args["post__not_in"] = array_filter(explode(',', $args['post__not_in']));
        }

        //      var_dump($args);



        // Handle distance and location
        if (!empty($args["city"])) {
            if (empty($args["radius"])) {

                $args["radius"] = intval($args["radius"]);

                $place = Nominatim::geocode($args["city"]);
                if ($place) {
                    $args["longitude"] = $place['lon'];
                    $args["latitude"] = $place['lat'];
                }


            } else {
                $args["radius"] = intval($args["radius"]);

                $place = Nominatim::geocode($args["city"]);
                if ($place) {
                    $args["longitude"] = $place['lon'];
                    $args["latitude"] = $place['lat'];
                }


                $posts = Nominatim::get_posts_in_distance($args["longitude"], $args["latitude"], $args["radius"],  $sort='asc');
                $args['distances'] =    $posts;
                $IDS = array_map(function ($post) {
                    return $post->ID;
                }, $posts);
                $args['post__in'] = empty($IDS) ? [-1] : $IDS;
          
            }



        }

        if (!empty($args["zip"])) {

            if (empty($args["radius"])) {
                // Handle company_id

                $args['meta_query'] = [
                    'relation' => 'AND',
                    ['key' => 'zip', 'value' => $args["zip"], 'compare' => '=']
                ];


            } else {
                $args["radius"] = intval($args["radius"]);

                $place = Nominatim::geocode($args["zip"]);
                if ($place) {
                    $args["longitude"] = $place['lon'];
                    $args["latitude"] = $place['lat'];

                    // Handle distance and location
                    if (!empty($args["radius"]) && !empty($args["longitude"]) && !empty($args["latitude"])) {
                        $posts = Nominatim::get_posts_in_distance($args["longitude"], $args["latitude"], $args["radius"],'asc');
                        $args['distances'] =    $posts;
                        $IDS = array_map(function ($post) {
                            return $post->ID;
                        }, $posts);
                        $args['post__in'] = empty($IDS) ? [-1] : $IDS;
                     
                    }
                }



            }
        }


        // Handle taxonomies
        $taxonomies = ['jobcategory', 'job_form_of_employment', 'job_working_time', 'working_time', 'job_type'];
        foreach ($taxonomies as $taxonomy) {
            if (!empty($args[$taxonomy])) {
                $term_ids = array_filter(explode(',', $args[$taxonomy]));
                if (!empty($term_ids)) {
                    $tax_query[] = [
                        'taxonomy' => $taxonomy,
                        'field' => 'term_id',
                        'terms' => $term_ids,
                        'operator' => 'IN'
                    ];
                }
            }

        }

        foreach ($taxonomies as $taxonomy) {
            $taxonomy_str = $taxonomy . '_str';



            if (!empty($args[$taxonomy_str])) {

                $tax_query[] = [
                    'taxonomy' => $taxonomy,
                    'field' => 'name',
                    'terms' => $args[$taxonomy_str],

                ];
            }


        }




        global $active_terms;
        if ($active_terms) {
            foreach ($active_terms as $taxonomy => $term_id) {


                $tax_query[] = [
                    'taxonomy' => $taxonomy,
                    'field' => 'term_id',
                    'terms' => $term_id,
                    'operator' => 'IN'
                ];



            }
        }




        if (count($tax_query) > 1) { // Ensure there are actual tax queries, besides the initial 'relation' element
            $args['tax_query'] = $tax_query;
        }

        // Handle dates
        if (!empty($args["dates"])) {
            $date_query = $this->get_date_query($args["dates"]);
            if ($date_query)
                $args['date_query'] = $date_query;
        }

        // Handle company_id
        if (!empty($args["company_id"])) {
            $args['meta_query'] = [
                'relation' => 'AND',
                ['key' => 'company_id', 'value' => $args["company_id"], 'compare' => '=']
            ];
        }

        if (isset($args['orderby']) && $args['orderby'] == 'rand') {
            $seed = isset($_COOKIE['random_seed']) ? $_COOKIE['random_seed'] : rand();

            $args["orderby"] = 'RAND(' . $seed . ')';
            $args["order"] = 'DESC';
        }

        if (isset($args['orderby']) && $args['orderby'] == 'distance') {
     
        
                $args['orderby']  = 'post__in';
        
        }

        return $args;
    }

    /**
     * Displays a div with the text "selected".
     *
     * @return void
     */
    public function show_selected_values()
    {
        echo '<div>selected</div>';
    }
    /**
     * Builds a query for retrieving posts based on the given parameters.
     *
     * @param array $params An array of parameters used to build the query.
     * @return array The built query arguments.
     */
    public function build_query($params)
    {
        $seed = isset($_COOKIE['random_seed']) ? $_COOKIE['random_seed'] : rand();

        $args = [
            'orderby' => 'RAND(' . $seed . ')',
            'order' => 'DESC',
        ];

        $args = shortcode_atts($args, $params);

        //   exit();


        $tax_query = [
            'relation' => 'AND',
        ];
        if (isset($args['orderby']) && $args['orderby'] == 'rand') {
            $seed = isset($_COOKIE['random_seed']) ? $_COOKIE['random_seed'] : rand();

            $args["orderby"] = 'RAND(' . $seed . ')';
            $args["order"] = 'DESC';
        }

        if (isset($params['s'])) {
            $args['s'] = $params['s'];
            //    $args['search_placeholder'] = $params['s'];
        }

        if (isset($params['city']) && isset($params['distance']) && $params['city'] && $params["distance"]) {
            if (isset($params['longitude']) && isset($params['latitude'])) {
                $args["longitude"] = $params['longitude'];
                $args["latitude"] = $params['latitude'];
            } else {

                $place = Nominatim::geocode($params["city"]);

                $args["longitude"] = $place['lon'];
                $args["latitude"] = $place['lat'];
            }
            $args["city"] = $params['city'];

            $posts = Nominatim::get_posts_in_distance($args["longitude"], $args["latitude"], $params["distance"], $sort='asc');
            $args['distances'] =    $posts;
            $IDS = array_map(function ($post) {
                return $post->ID;
            }, $posts);
            $args['post__in'] = empty($IDS) ? [-1] : $IDS;
            if (isset($args['orderby']) && $args['orderby'] == 'distance') {
                $args['orderby']  = 'post__in';
            }
        }

        $categories = [
            'jobcategory',
            'job_form_of_employment',
            'job_working_time',
            'job_type',
        ];


        /* 
         * use select_ as prefix for the select fields
         * foraall select_ fields check if they are set
         * if set, add them to the tax_query with an OR opeerator
         */
        $selected_terms = [
            'relation' => 'OR',

        ];

        foreach ($categories as $category) {
            $key = 'select_' . $category;

            if (isset($params[$key])) {

                foreach ($params[$key] as $term_id) {

                    $term_query = [
                        'taxonomy' => $category,
                        'field' => 'term_id',
                        'terms' => $term_id,


                    ];
                    $selected_terms[] = $term_query;
                }




            }

        }
        $tax_query[] = $selected_terms;



        if (isset($params["distance"]) && !empty($params["longitude"]) && !empty($params["latitude"])) {
            $posts = Nominatim::get_posts_in_distance($params["longitude"], $params["latitude"], $params["distance"], $sort='asc');
            $args['distances'] =    $posts;
            $IDS = [];
            foreach ($posts as $post) {
                $IDS[] = $post->ID;
            }
            if (count($IDS) == 0) {
                $IDS = array(-1);
            } else {
            }
            $args['post__in'] = $IDS;

        }

        if (isset($params["jobcategory"])) {

            $term_ids = explode(',', $params['jobcategory']);
            $term_ids = array_filter($term_ids);

            $term_query = [
                'taxonomy' => 'jobcategory',
                'field' => 'term_id',
                'terms' => $term_ids,
                //  'operator' => 'IN'

            ];
            $tax_query[] = $term_query;

        }


        if (isset($params["job_type"])) {

            $term_ids = explode(',', $params['job_type']);
            $term_ids = array_filter($term_ids);
            $term_query = [
                'taxonomy' => 'job_type',
                'field' => 'term_id',
                'terms' => $term_ids,
                'operator' => 'IN',
            ];
            $tax_query[] = $term_query;



        }

        if (isset($params["working_time_terms"])) {

            $term_ids = explode(',', $params['working_time_terms']);
            $term_ids = array_filter($term_ids);
            $term_query = [
                'taxonomy' => 'job_working_time',
                'field' => 'term_id',
                'terms' => $term_ids,
                'operator' => 'IN',
            ];
            $tax_query[] = $term_query;

        }

        if (isset($params["form_of_employment_terms"])) {

            $term_ids = explode(',', $params['form_of_employment_terms']);
            $term_ids = array_filter($term_ids);


            $term_query = [
                'taxonomy' => 'job_form_of_employment',
                'field' => 'term_id',
                'terms' => $term_ids,
                'operator' => 'IN',
            ];
            $tax_query[] = $term_query;

        }


        if (count($tax_query) > 1) {
            $args['tax_query'] = $tax_query;
        }



        if (isset($params["dates"]) && !empty($params["dates"])) {
            if ($this->get_date_query($params["dates"])) {
                $args['date_query'] = $this->get_date_query($params["dates"]);
            }

        }
        if (!empty($params["company_id"])) {
            $args['meta_query'] = [
                'relation' => 'AND',
                ['key' => 'company_id', 'value' => $params["company_id"], 'compare' => '=']
            ];
        }


        if (isset($params["activeTermTaxonomy"])) {
            $args['tax_query']['relation'] = 'AND';
        }



        return $args;
    }
    /**
     * Retrieves job listings based on the given request parameters.
     *
     * @param array $request The request parameters.
     * @return array The job listings.
     */
    public function get_jobs($request)
    {
        $params = $request->get_params($request);
        if ($params['city']) {
            $place = Nominatim::geocode($params["city"]);
            $args["longitude"] = $place['lon'];
            $args["latitude"] = $place['lat'];
            $posts = Nominatim::get_posts_in_distance($args["longitude"], $args["latitude"], $args["radius"], $sort='asc');
            $IDS = array_map(function ($post) {
                return $post->ID;
            }, $posts);
            $args['post__in'] = empty($IDS) ? [-1] : $IDS;
        }


       if($args['orderby'] == 'distance') {
            $args['orderby']  = 'post__in';
        }

        $args = $this->build_query($params);

        $args["radius"] = intval($params["distance"]);
        $args['posts_per_page'] = $request['posts_per_page'];
        //$args['paged'] = $request['page'];
        $place = Nominatim::geocode($args["city"]);

        $content = Filter::get_results($args);
        return $content;

    }
    /**
     * Returns the reverse date query based on the given time period.
     *
     * @param string $after The time period to base the reverse date query on.
     * @return string The reverse date query.
     */
    public static function get_reverse_date_query($after)
    {
        switch ($after) {
            case 'today':
                $period = 'today';
                break;
            case '3 days ago':
                $period = '-3';
                break;
            case '7 days ago':
                $period = '-7';
                break;
            case '14 days ago':
                $period = '-14';
                break;

            case '30 days ago':
                $period = '-30';
                break;
            default:
                $period = '-9000';
                break;
        }
        return $period;
    }
    /**
     * Returns a date query array based on the given period.
     *
     * @param string $period The period for the date query.
     * @return array|bool The date query array if a valid period is given, false otherwise.
     */
    function get_date_query($period)
    {
        switch ($period) {
            case 'today':
                $date_query = array(
                    'after' => 'today',
                    'before' => 'tomorrow',
                    'inclusive' => true,
                );
                break;
            case '-3':
                $date_query = array(
                    'after' => '3 days ago',
                    'before' => 'tomorrow',
                    'inclusive' => true,
                );
                break;
            case '-7':
                $date_query = array(
                    'after' => '7 days ago',
                    'before' => 'tomorrow',
                    'inclusive' => true,
                );
                break;
            case '-14':
                $date_query = array(
                    'after' => '14 days ago',
                    'before' => 'tomorrow',
                    'inclusive' => true,
                );
                break;
            case '-30':
                $date_query = array(
                    'after' => '30 days ago',
                    'before' => 'tomorrow',
                    'inclusive' => true,
                );
                break;
            default:
                $date_query = False;
                break;
        }
        return $date_query;
    }
    /**
     * Retrieves the parameters from the global $wp_query object.
     *
     * @return array The parameters from the global $wp_query object.
     */
    private function get_params()
    {
        global $wp_query;
        return $wp_query->query;
    }

    private function get_selected_terms($terms, $bys, $values)
    {
        $r_terms = [];
        foreach ($terms as $i => $term) {

            $r_terms[] = get_term_by($bys[$i], $values[$i], $terms[$i]);
        }


        return $r_terms;
    }

    private function combinate_selected_terms($terms, $bys, $values)
    {

        $selected_terms = [
            'relation' => 'OR',

        ];


        foreach ($terms as $i => $term) {
            $t = get_term_by($bys[$i], $values[$i], $terms[$i]);
            $term_query = [
                'taxonomy' => $term,
                'field' => 'term_id',
                'terms' => $t->term_id,


            ];
            $selected_terms[] = $term_query;
        }

        return $selected_terms;
    }
    /**
     * Generates a search form shortcode.
     *
     * @return string The generated search form shortcode.
     */
    public function search_form_shortcode($atts)
    {
        $defaults = array(

            'id' => uniqid(),
            'post_type' => 'job',
            'post_status' => 'publish',
            'per_page' => '10',
            'order' => 'asc',
            'posts_per_page' => -1,
            'orderby' => 'date',
            'jobcategory' => '',
            'job_type' => '',
            'radius' => false,
            'city' => false,
            'zip' => false,
            'job_form_of_employment_str' => false,
            'job_working_time_str' => false,
            'job_type_str' => false,
            'premium' => false,
            'company_id' => false,
            'post__not_in' => '',
            's' => false,
            'show_pagination' => false,
            'placeholder' => '',
            'city_placeholder' => '',
            'terms' => [],
            'bys' => [],
            'values' => [],
        );

        $atts = shortcode_atts($defaults, $atts);
        if (isset($atts['terms'])) {
            if (is_string($atts['terms'])) {
                $atts['terms'] = explode(',', $atts['terms']);
            }
        }
        if (isset($atts['bys'])) {
            if (is_string($atts['bys'])) {
                $atts['bys'] = explode(',', $atts['bys']);
            }
        }


        if (isset($atts['values'])) {
            if (is_string($atts['values'])) {
                $atts['values'] = explode(',', $atts['values']);
            }
        }

        global $wp_query;

        ob_start();

        ?>
        <form class="container dp-form dp-form-<?php echo $atts['id']; ?>">

            <div class="dynamic-posts">
                <div class="filter" id="dp-filter">

                    <?php echo Filter::show_filter($wp_query->query, $atts); ?>

                </div>


            </div>
        </form>
        <?php
        $content = ob_get_contents();
        ob_end_clean();

        return $content;
    }
    /**
     * Renders a complete filter with the given attributes and content. 
     *
     * Shortcode [filter_jobs] 
     *
     * This shortcode is used to create a full-featured search filter for job posts. 
     * It provides configuration for sorting, pagination, showing more buttons and others. 
     * @ToDo render job results with Premium status separately.
     * 
     * @since 1.0.0
     *
     * @shortcode [filter_jobs posts_per_page="" show_more_button="" orderby="" order="" 
     *                          premium="" company_id="" s="" show_pagination="" show_order=""]
     *
     * @param array $atts {
     *     An array of attributes used by the shortcode.
     *
     *     @type int    $posts_per_page The number of posts to be displayed per page. Default '10'.
     *     @type bool   $show_more_button Whether to display a 'show more' button. Default 'False'.
     *     @type string $orderby The parameter to order the posts by. Default 'rand'.
     *     @type string $order The order to display the posts in. Default 'DESC'.
     *     @type bool   $premium Whether to display premium posts. Default 'False'.
     *     @type bool   $company_id Whether to filter posts by company ID. Default 'False'.
     *     @type string $s The search query. Default 'False'.
     *     @type bool   $show_pagination Whether to show pagination. Default 'False'.
     *     @type bool   $show_order Whether to show order sort. Default 'False'.
     * }
     *
     * @return string The rendered filter HTML output.
     */


    function get_all_used_shortcodes()
    {
        global $wp_filter;
        global $shortcode_tags;
        global $page;
        global $post;

    }

    public function render_complete_filter($atts = [], $content = null)
    {
        $seed = isset($_COOKIE['random_seed']) ? $_COOKIE['random_seed'] : rand();

        $defaults = array(
            'posts_per_page' => 10,
            'show_more_button' => False,
            'orderby' => 'RAND(' . $seed . ')',
            'order' => 'DESC',
            'premium' => false,
            'company_id' => false,
            's' => false,
            'show_pagination' => false,
            'show_order' => false,
            'id' => uniqid(),
            'post_type' => 'job',
            'post_status' => 'publish',
            'per_page' => '10',
            'jobcategory' => '',
            'job_type' => '',
            'radius' => false,
            'city' => false,
            'zip' => false,
            'job_form_of_employment_str' => false,
            'job_working_time_str' => false,
            'job_type_str' => false,
            'post__not_in' => '',
            'terms' => [],
            'bys' => [],
            'values' => [],
            'placeholder' => '',
            'city_placeholder' => '',
        );




        $sh_args = shortcode_atts($defaults, $atts);


        $shortcodes = Shortcodes::get_shortcode_global_atts();
        if (isset($shortcodes['id'])) {
            $sh_args = shortcode_atts($sh_args, $shortcodes['id']);
        }




        if (isset($sh_args['terms'])) {
            if (is_string($sh_args['terms'])) {
                $sh_args['terms'] = explode(',', $sh_args['terms']);
            }

        }
        if (isset($sh_args['bys'])) {
            if (is_string($sh_args['bys'])) {
                $sh_args['bys'] = explode(',', $sh_args['bys']);
            }

        }


        if (isset($sh_args['values'])) {
            if (is_string($sh_args['values'])) {
                $sh_args['values'] = explode(',', $sh_args['values']);
            }
        }


        $form_id = $sh_args["id"];

        $params = $this->get_params();
        $sh_args = $this->build_args($sh_args);
        $args = $this->build_query($params);
        $bar_args = $args;
        $args = array_merge(
            $args,
            $sh_args,
            [
                "tax_query" => $this->combinate_selected_terms($sh_args['terms'], $sh_args['bys'], $sh_args['values'])
            ],
        );


        $params['posts_per_page'] = $sh_args['posts_per_page'];
        $args['posts_per_page'] = $sh_args['posts_per_page'];
        if (isset($atts['orderby'])) {
            $args['orderby'] = $atts['orderby'];
            $args['order'] = $atts['order'];
   
            if ($atts['orderby'] == 'date') {
                $args['orderby'] =  'meta_value_num';
                $args['meta_key'] =  'job_start_date';
                $args['order'] = $atts['order'];
            }
            if($args['orderby'] == 'distance') {
        
                $args['orderby']  = 'post__in';
            }
        }
        $this->query = new \WP_Query($this->build_args($params));

        ob_start();
        global $wp_query;



        $LGenQuery = new LGenQuery();
        $args = $LGenQuery->build_args($args);
   
 
        $query = new WP_Query($args);
        $posts_per_page = $query->query_vars['posts_per_page'];
        $paged = $query->query_vars['paged'];
        $found_posts = $query->found_posts;
        $max_num_pages = $query->max_num_pages;


        $args['tax_query']['relation'] = 'AND';




        ?>
        <form class="container dp-form dp-form-<?php echo $sh_args['id']; ?>">
            <input type="hidden" id="latitude" value="<?php if (isset($sh_args['latitude']))
                echo $sh_args['latitude']; ?>" name="latitude">
            <input type="hidden" id="longitude" value="<?php if (isset($sh_args['longitude']))
                echo $sh_args['longitude']; ?>" name="longitude">

            <div class="dynamic-posts">
                <!--      <div class="filter" id="dp-filter">

                </div>
    -->
                <div class="sidebar">
                    <?php echo Filter::show_sidebar($args); ?>
                </div>
                <div class="content">
                    <div class="uk-grid uk-flex uk-flex-middle" uk-grid>

                        <div class="uk-width-1-1 uk-width-3-4@m uk-flex">
                            <!-- First column content -->

                            <div id="filter_selected_bar">

                                <?php echo FilterController::render_bar(FilterController::parse_bar_args($args, $params), $found_posts); ?>

                            </div>
                        </div>
                        <?php
                        if (isset($atts['show_order'])) {
                            ?>
                            <div class="uk-width-1-1 uk-width-1-4@m">
                                <!-- Second column content -->
                                <?php echo Filter::show_sorting(); ?>
                            </div>


                            <?php
                        } else {
                            ?>
                            <input type="hidden" name="orderby" value="date" class="uk-select">
                            <input type="hidden" name="order" value="desc" class="uk-select">
                            <?php
                        } ?>

                    </div>

                    <div id="posts_content_premium_wrap">
                        <div id="dp-premium-output" class="main-box show all_article">
                            <div class="job-wrapper">

                                <?php echo Filter::get_premium_results($args); ?>

                            </div>
                        </div>
                    </div>

                    <div id="posts_content_wrap">
                        <div id="dp-output-<?php echo $form_id; ?>" class="main-box show all_article">
                            <div class="job-wrapper">

                                <?php echo Filter::get_job_results($args); ?>

                            </div>
                        </div>
                    </div>

                </div>
            </div>
        </form>
        <?php
        ?>
        <?php if (isset($atts['show_more_button']) && $atts['show_more_button'] != False) {
            ?>
            <div class="job_archive_link_wrapper uk-text-center">
                <a class="job_archive_link uk-button uk-button-secondary uk-flex-middle"
                    href="<?php echo get_post_type_archive_link('job'); ?>">
                    <span class="uk-margin-small-right uk-icon">
                        <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" width="16">
                            <path
                                d="M40 48C26.7 48 16 58.7 16 72v48c0 13.3 10.7 24 24 24H88c13.3 0 24-10.7 24-24V72c0-13.3-10.7-24-24-24H40zM192 64c-17.7 0-32 14.3-32 32s14.3 32 32 32H480c17.7 0 32-14.3 32-32s-14.3-32-32-32H192zm0 160c-17.7 0-32 14.3-32 32s14.3 32 32 32H480c17.7 0 32-14.3 32-32s-14.3-32-32-32H192zm0 160c-17.7 0-32 14.3-32 32s14.3 32 32 32H480c17.7 0 32-14.3 32-32s-14.3-32-32-32H192zM16 232v48c0 13.3 10.7 24 24 24H88c13.3 0 24-10.7 24-24V232c0-13.3-10.7-24-24-24H40c-13.3 0-24 10.7-24 24zM40 368c-13.3 0-24 10.7-24 24v48c0 13.3 10.7 24 24 24H88c13.3 0 24-10.7 24-24V392c0-13.3-10.7-24-24-24H40z"
                                fill="%2300466c" />
                        </svg>
                    </span>
                    Alle Jobs anzeigen
                </a>
            </div>
            <?php
        } else {
            echo self::get_pagination($posts_per_page, $paged, $found_posts, $max_num_pages, $form_id);


        }?>


        </div>
        <?php
        $content = ob_get_contents();
        ob_end_clean();

        return $content;
    }



}
class FilterController extends Filter
{
    private $namespace;
    /**
     * Constructor for the class.
     * Initializes the distance fields and date times arrays.
     * Adds shortcodes for rendering the filter, sorting, counting, selected filters, search form, and pagination.
     * Adds actions for loading scripts and handling AJAX callbacks.
     */
    /**
     * Constructor for the Leadgen class.
     * Initializes the namespace and registers REST routes for job listings and address search.
     *
     * @since 1.0.0
     */
    function __construct()
    {


        $this->namespace = 'leadgen';
        add_action('rest_api_init', function () {
            register_rest_route($this->namespace . '/v1/jobs', 'jobs', [
                'methods' => WP_REST_Server::READABLE,
                'callback' => [$this, 'get_jobs'],
                'permission_callback' => function ($request) {
                    return true;


                },
                'args' => array(
                    'page' => array(
                        //  'validate_callback' => 'is_numeric',
                        'default' => 1,
                    ),
                    'per_page' => array(
                        //   'validate_callback' => 'is_numeric',
                        'default' => 10,
                    ),

                    'args' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
            ]);
            register_rest_route($this->namespace . '/v1/jobs/address', 'search', [
                'methods' => WP_REST_Server::READABLE,
                'callback' => [$this, 'call_openstreet_map'],
                'permission_callback' => function ($request) {
                    return true;


                },
                'args' => array(
                    'args' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
            ]);
            register_rest_route($this->namespace . '/v1/jobs/address', 'autocomplete', [
                'methods' => WP_REST_Server::READABLE,
                'callback' => [$this, 'call_autocomplete_cities'],
                'permission_callback' => function ($request) {
                    return true;
                },
                'args' => array(
                    'args' => array(
                        'validate_callback' => function ($param, $request, $key) {
                            return is_numeric($param);
                        }
                    ),
                ),
            ]);

            register_rest_route($this->namespace . '/v1/jobs/filter', 'sidebar', [
                'methods' => WP_REST_Server::READABLE,
                'callback' => [$this, 'update_sidebar'],
                'permission_callback' => function ($request) {
                    return true;
                }

            ]);
            register_rest_route($this->namespace . '/v1/jobs/filter', 'bar', [
                'methods' => WP_REST_Server::READABLE,
                'callback' => [$this, 'update_bar'],
                'permission_callback' => function ($request) {
                    return true;
                }

            ]);

        });

    }

    public static function processTaxQuery($queries, &$values)
    {

        foreach ($queries as $query) {
            if (isset($query['taxonomy']) && isset($query['terms'])) {
                $taxonomy = $query['taxonomy'];
                $terms = $query['terms'];

                if (is_numeric($terms)) {


                    $term_id = get_term_by('term_id', $terms, $taxonomy)->term_id;
                    $values[$taxonomy][$term_id] = get_term($term_id, $taxonomy)->name;
                }
                if (is_string($terms) && !is_numeric($terms)) {

                    $term_id = get_term_by('slug', $terms, $taxonomy)->term_id;
                    $values[$taxonomy][$term_id] = get_term($term_id, $taxonomy)->name;
                }
                if (is_array($terms)) {
                    foreach ($terms as $term) {

                        $term_id = get_term_by('term_id', $term, $taxonomy)->term_id;
                        $values[$taxonomy][$term_id] = get_term($term_id, $taxonomy)->name;
                    }
                }


                if (is_array($terms)) {

                } else {

                }

            }

            // Recursively check nested queries
            if (isset($query['relation']) && is_array($query)) {
                self::processTaxQuery($query, $values);
            }
        }
    }

    /**
     * Parses the arguments for a bar and returns the relevant values.
     *
     * @param array $args The arguments for the bar.
     * @param array $params The parameters for the bar.
     * @return array The parsed values for the bar.
     */
    public static function parse_bar_args($args, $params)
    {


        $values = [];

        // Order fields
        $orderFields = ['orderby', 'order', 's'];
        foreach ($orderFields as $field) {
            if (isset($args[$field])) {
                // Escape search query
                $values[$field] = $field === 's' ? htmlspecialchars($args[$field]) : $args[$field];
            }
        }

        if (isset($args['city'])) {
            $values['city'] = $args['city'];
        }


        if (isset($args['tax_query']) && is_array($args['tax_query'])) {
            self::processTaxQuery($args['tax_query'], $values);
        }


        $dateMap = [
            "today" => "Heute",
            "3 days ago" => "letzten 3 Tage",
            "7 days ago" => "letzten 7 Tage",
            "14 days ago" => "letzten 14 Tage",
            "30 days ago" => "letzten 30 Tage",
        ];

        // Date query
        if (isset($args['date_query']) && is_array($args['date_query'])) {
            $values['dates'][$params['dates']] = $dateMap[$args['date_query']["after"]];

        }



        return $values;
    }
    /**
     * Counts the number of posts associated with a given term ID.
     *
     * @param int $term_id The ID of the term to count posts for.
     * @param array $args Optional. Additional arguments for the WP_Query object.
     * @return int The number of posts associated with the given term ID.
     */
    public function count_term_by_id($term_id, $args)
    {
        $query = new WP_Query($args);

        return $query->found_posts;
    }
    /**
     * Renders a bar selection using the given arguments.
     *
     * @param array $args The arguments to be used in rendering the bar selection.
     * @return string The rendered HTML for the bar selection.
     */
    public static function render_bar_selections($args)
    {


        $data = [
            'values' => $args,
        ];
        $template = 'filter/bar/selection.twig';

        $rendered_html = Timber::compile($template, $data);
        return $rendered_html;
    }
    /**
     * Renders a bar filter with the given arguments and count.
     *
     * @param array $args The arguments to be used in the filter.
     * @param int $count The count of items to be displayed in the filter.
     * @return string The rendered HTML of the bar filter.
     */
    public static function render_bar($args, $count)
    {



        $data = [
            'values' => $args,
            'count' => $count,
            'text' => get_option('leadgen_filter_text', 'Jobs in')
        ];
        if (isset($args['city']) && !empty($args['city'])) {
            $data['text'] = 'Jobs in ' . $args['city'];
        }
        if (isset($args['orderby']) && $args['orderby'] == 'rand') {

        }
        $template = 'filter/bar/bar.twig';

        $data = self::array_filter_recursive($data);

        $rendered_html = Timber::compile($template, $data);
        return $rendered_html;
    }
    /**
     * Updates the bar with the given request parameters.
     *
     * @param WP_REST_Request $request The request object containing the parameters.
     * @return array An array containing the updated HTML and arguments.
     */
    public function update_bar($request)
    {

        $params = $request->get_params($request);

        $args = $this->build_query($params);

        $jobQuery = new LGenQuery();

        $q_args = $jobQuery->build_args($args);
        $the_query = new WP_Query($args);
        // $count = $the_query->found_posts;

        $args = $this->parse_bar_args($args, $params);



        return [
            'html' => self::render_bar_selections($args),
            'args' => $args
        ];

    }

    static function array_filter_recursive($array)
    {
        foreach ($array as $key => $value) {
            if (is_array($value)) {
                $array[$key] = self::array_filter_recursive($value);
            }
        }

        return array_filter($array, function ($subArray) {
            return is_array($subArray) ? self::array_filter_recursive($subArray) : $subArray;
        });
    }


    /**
     * Updates the sidebar with the given request parameters.
     *
     * @param $request The request object containing the parameters.
     * @return array An array containing the calculated term counts for each taxonomy.
     */
    public function update_sidebar($request)
    {
        $params = $request->get_params($request);


        $args = $this->build_query($params);


        $jobQuery = new LGenQuery();

        $args = $jobQuery->build_args($args);

        // Array of taxonomies
        $taxonomies = [
            'jobcategory',
            'job_type',
            'job_working_time',
            'job_form_of_employment',
            //     'job_working_time_terms',
        ];
        $keysFromParams = array_keys($params);
        $result = array_diff($taxonomies, $keysFromParams);



        // Assume $params is your given array
        foreach ($params as $key => $value) {

            if (in_array($key, $taxonomies)) {
                $taxonomyKey = array_search($key, $taxonomies);


                if ($taxonomyKey != 0) {
                    unset($taxonomies[$taxonomyKey]);
                }

            }
        }


        $periods = [
            'today',
            '-3',
            '-7',
            '-14',
            '-30',
            '-9000',

        ];



        $result = $this->count_jobs_by_periods($periods, $args);


        $counts = [
            'dates' => $result,
        ];

        foreach ($taxonomies as $taxonomy) {

            $counts[$taxonomy] = $this->calculate_term_counts($args, $taxonomy);

        }

        return $counts;
    }

    function count_jobs_by_periods($periods, $args)
    {
        $counts = [];

        foreach ($periods as $period) {

            $counts[$period] = $this->calculate_date_counts($args, $period);
        }

        return $counts;
    }


    /**
     * Calculates the term counts for a given taxonomy and arguments.
     *
     * @param array $args The arguments to be used in the term count calculation.
     * @param string $taxonomy The taxonomy to be used in the term count calculation.
     * @return array An array of term counts, with the term ID as the key and the count as the value.
     */
    private function calculate_term_counts($args, $taxonomy)
    {
        // Here the unique key is created using the function's arguments 
        // $transient_key = 'term_counts_' . md5(serialize($args) . $taxonomy);
        //$cached_result = get_transient($transient_key);

        // If the transient exists, return the cached data
        /*if ($cached_result !== false) {
            return $cached_result;
        }
*/
        $term_ids = get_terms([
            'taxonomy' => $taxonomy,
            'hide_empty' => false,
            'fields' => 'ids'
        ]);

        $counts = [];
        $original_tax_query = $args["tax_query"] ?? [];

        foreach ($term_ids as $id) {

            $args["tax_query"] = $original_tax_query;
            $args["tax_query"][] = [
                [
                    'taxonomy' => $taxonomy,
                    'field' => 'term_id',
                    'terms' => $id
                ]
            ];



            $counts[$id] = $this->count_term_by_id($id, $args);
        }

        // Cache the result
        //   set_transient($transient_key, $counts, DAY_IN_SECONDS);

        return $counts;
    }

    private function calculate_date_counts($args, $period)
    {
        $args['posts_per_page'] = -1;
        $args['date_query'] = $this->get_date_query($period);


        $jobs = get_posts($args);
        $count = count($jobs);

        return $count;
    }


}
//new Filter();
new FilterController();

add_action('rest_api_init', function () {

    register_rest_field(
        'job',
        'rendered_job_item',
        array(
            'get_callback' => function ($post_arr) {
                $post = get_post($post_arr['id']);
                $LeadPost = new LGenPost(); //Created instance of the LGenPost class
                $job = $LeadPost->setup_job($post);
                return $LeadPost->render_job_item($job); //Return the result of render_job_item method for the current job post
            },
            'update_callback' => null,
            'schema' => null,
        )
    );

});



/**
 * Filters the query arguments for a REST request.
 *
 * @param array $args The query arguments.
 * @param WP_REST_Request $request The REST request object.
 * @return array The filtered query arguments.
 */
function filter_rest_query($args, $request)
{
    $params = $request->get_query_params();
    $filter = new Filter();


    // If 'page' is set in the request, use that for 'paged'
    if ($request->get_param('page')) {
        $args['paged'] = $request->get_param('page');
    } else {
        // If 'paged' is not passed in the request, default it to 1
        $args['paged'] = 1;
    }

    if ($params) {

        $constructed_args = $filter->build_query($params);
        $args = array_merge($args, $constructed_args);

    }

    return $args;
}

add_filter('rest_job_query', 'filter_rest_query', 10, 2);


