<?php
/**
 * Class Account
 * This class is used to handle the companies inside a Lead Generation system on WordPress. 
 * The class provides methods to check if a company profile exists, retrieve companies, 
 * get user IDs, load necessary scripts as well as handle other related functionalities.  
 * 
 * It also includes actions and filters to interact with the WooCommerce checkout fields, 
 * account menus, and account user interface. 
 * @package LeadGenerator
 *
 * @since 0.2.1
 */

defined('ABSPATH') || exit;
class Account
{
    public const Fields = [
        'signet',
        'unternehmensgrose',
        'bennefits',
        'branche',
        'phone',
        'email',
        'street',
        'zip',
        'city',
        'country'
    ];
    /**
     * Constructs a new instance of the class.
     *
     * This method is used to initialize a new instance of the class and load any necessary scripts.
     *
     * @return void
     */
    public function __construct()
    {

        $this->load_scripts();

    }
    /**
     * Checks if a company profile exists for a given user ID.
     *
     * @param int $user_id The ID of the user to check for a company profile.
     * @return array An array of company posts if a profile exists, otherwise an empty array.
     */
    public static function check_company_profil($user_id)
    {
        $args = array(
            'posts_per_page' => -1,
            'post_type' => 'company',

            'meta_query' => array(
                array(
                    'key' => 'user_id',
                    'value' => $user_id,
                    'compare' => '=',
                )
            )
        );
        $posts = get_posts($args);
        if (count($posts) > 0) {
            return $posts;
        } else {
            return [];
        }

    }
    /**
     * Retrieves all companies associated with a given user ID.
     *
     * @param int $user_id The ID of the user to retrieve companies for.
     * @return array An array of company post objects, or an empty array if no companies are found.
     */
    public static function get_companies($user_id)
    {
        $args = array(
            'posts_per_page' => -1,
            'post_type' => 'company',
            'meta_key' => 'user_id',
            'meta_query' => array(
                array(
                    'key' => 'user_id',
                    'value' => $user_id,
                    'compare' => '=',
                )
            )
        );
        $posts = get_posts($args);
        if (count($posts) > 0) {
            return $posts;
        } else {
            return [];
        }

    }
    /**
     * Retrieves the user ID associated with a given company ID.
     *
     * @param int $company_id The ID of the company to retrieve the user ID for.
     * @return int|null The user ID associated with the given company ID, or null if no user ID is found.
     */
    public static function get_user_id_from_company_id($company_id)
    {
        $user_id = get_post_meta($company_id, 'user_id', true);
        if ($user_id > 0) {
            return $user_id;
        }
    }
    /**
     * Loads the necessary scripts for the form autofill functionality.
     *
     * @return void
     */
    public function load_scripts()
    {
        wp_register_script('autofill', LGenURL . 'dist/form-autofill.js', ['wp-api', 'wp-api-fetch', 'wp-i18n', 'underscore', 'backbone', 'wp-dom-ready'], []);
        wp_enqueue_script('autofill');

        global $current_user;
        $current_user = wp_get_current_user();

        // Execute your code for the customer
        $user_id = $current_user->ID;
        wp_localize_script(
            'autofill',
            'args',
            [
                'user_id' => $user_id
            ]
        );




    }

}

add_action('init', function () {
    new Account();
});



add_filter('woocommerce_checkout_fields', 'custom_override_checkout_fields');
/**
 * Removes the shipping field from the checkout fields array.
 *
 * @param array $fields The array of checkout fields.
 * @return array The updated array of checkout fields with the shipping field removed.
 */
function custom_override_checkout_fields($fields)
{
    unset($fields['shipping']);
    return $fields;
}
/**
 * Adds the custom query variable 'myjobs' to the list of query variables.
 *
 * @param array $vars The list of query variables.
 * @return array The updated list of query variables with 'myjobs' added.
 */
function custom_endpoint_query_vars($vars)
{
    $vars[] = 'myjobs';
    return $vars;
}
add_filter('query_vars', 'custom_endpoint_query_vars', 0);
/**
 * Adds a custom menu item to the "My Account" menu.
 *
 * @param array $items The current list of menu items.
 * @return array The updated list of menu items.
 */
function my_custom_my_account_menu_items($items)
{
    $items['myjobs'] = __('Meine Anzeigen', 'woocommerce');
    return $items;
}
add_filter('woocommerce_account_menu_items', 'my_custom_my_account_menu_items', 10, 1);
/**
 * Adds custom endpoints for "myjobs" and "myprofil" to the WordPress rewrite rules.
 *
 * @since 1.0.0
 *
 * @return void
 */
function myjobs_woo_custom_tab_endpoint()
{
    add_rewrite_endpoint('myjobs', EP_ROOT | EP_PAGES);
    add_rewrite_endpoint('myprofil', EP_ROOT | EP_PAGES);
}

add_action('init', 'myjobs_woo_custom_tab_endpoint');
/**
 * Retrieves and displays the custom endpoint content for the current user.
 *
 * @return void
 */
function custom_endpoint_content()
{
    $current_user = wp_get_current_user();



    $args = array(
        'posts_per_page' => -1,
        'post_type' => 'job', // replace this with your custom post type
        'orderby' => 'date',
        'order' => 'DESC',
        'post_status' => ['publish'],
        'meta_query' => array(
            array(
                'key' => 'user_id', //replace this with your custom field meta key
                'value' => $current_user->ID,
                'compare' => '=',
            )
        )
    );

    $my_jobs = new WP_Query($args);

    $context = Timber::context();


    $context['my_jobs'] = $my_jobs->posts;
    $context['my_jobs'] = array_map(function ($job) {
        return Timber::get_post($job->ID);
    }, $my_jobs->posts);


    $template = apply_filters('wp_leadgen_template_path', 'accounts/job_template.twig');
    $rendered_html = Timber::compile($template, $context);
    echo $rendered_html;


}
add_action('woocommerce_account_myjobs_endpoint', 'custom_endpoint_content');




add_filter('woocommerce_account_menu_items', 'remove_my_account_links');
/**
 * Removes the "Downloads" link from the "My Account" menu.
 *
 * @param array $menu_links The array of menu links for the "My Account" menu.
 * @return array The updated array of menu links with the "Downloads" link removed.
 */
function remove_my_account_links($menu_links)
{

    unset($menu_links['downloads']);  // Remove Downloads

    return $menu_links;
}
/**
 * Flushes the rewrite rules to ensure that any changes made to the permalinks are reflected.
 *
 * @return void
 */
function custom_flush_rewrite_rules()
{
    flush_rewrite_rules();
}
add_action('after_switch_theme', 'custom_flush_rewrite_rules');
/**
 * Adds the 'myprofil' query variable to the list of query variables.
 *
 * @param array $vars The current list of query variables.
 * @return array The updated list of query variables with 'myprofil' added.
 */
function myprofil_endpoint_query_vars($vars)
{
    $vars[] = 'myprofil';
    return $vars;
}
add_filter('query_vars', 'myprofil_endpoint_query_vars', 0);
/**
 * Adds a new menu item for the user's company profile in the "My Account" menu.
 *
 * @param array $items The existing menu items in the "My Account" menu.
 * @return array The updated menu items with the new company profile item.
 */
function my_profil_my_account_menu_items($items)
{
    $items['myprofil'] = __('Unternehmensprofil', 'woocommerce');
    return $items;
}
add_filter('woocommerce_account_menu_items', 'my_profil_my_account_menu_items', 10, 1);

//add_action('acf/save_post', 'form_companies_save_post');
/**
 * Saves the post data for a company.
 *
 * @param int $post_id The ID of the post being saved.
 */
function form_companies_save_post($post_id)
{
    if (get_post_type($post_id) != 'company') {
        return;
    }
    // Get newly saved values.
    $values = get_fields($post_id);

    // Check if address data is set
    if (isset($values['street']) && isset($values['city']) && isset($values['zip']) && isset($values['country'])) {

        // Build address string
        $address = $values['street'] . ', ' . $values['city'] . ', ' . $values['zip'] . ', ' . $values['country'];

        // Get geocode data from Nominatim
        $data = Nominatim::geocode($address);

        // Assuming the data array has longitude and latitude keys
        if (isset($data['longitude']) && isset($data['latitude'])) {

            // Update meta data's longitude and latitude values
            update_post_meta($post_id, 'longitude', $data['longitude']);
            update_post_meta($post_id, 'latitude', $data['latitude']);
        }
    }

}
/**
 * Retrieves the company profile endpoint for the WooCommerce account page.
 *
 * @param void
 * @return void
 */
function woocommerce_account_profil_endpoint()
{

    $current_user = wp_get_current_user();
    $company_id = isset($_GET['post_id']);

    $company_profiles = Account::get_companies($current_user->ID);
    $template = apply_filters('wp_leadgen_template_path', 'form/fields/companies.twig');
    $context = Timber::context();
    $context['companies'] = array_map(function ($company_profile) {
        return Timber::get_post($company_profile->ID);
    }, $company_profiles);
    $rendered_html = Timber::compile($template, $context);
    echo $rendered_html;


    // Converting company profiles into array of IDs
    $company_ids = array_map(function ($company_profile) {
        return $company_profile->ID;
    }, $company_profiles);
    $form_id = get_option('leadgen_cf7_company_registration_form_id');



    if ($company_id && in_array($company_id, $company_ids)) {
        $form = do_shortcode("[contact-form-7 id='" . $form_id . "']");
        echo $form;
    } elseif ($company_id == 'new') {
        $form = do_shortcode("[contact-form-7 id='" . $form_id . "']");
        echo $form;
    }

}
add_action('um_after_email_confirmation', 'um_after_email_confirmation_admin_approval', 99, 1);
/**
 * Updates the account status to 'awaiting_admin_review' after email confirmation and sends an email to the admin for approval.
 *
 * @param int $user_id The ID of the user whose account status will be updated.
 *
 * @return void
 */
function um_after_email_confirmation_admin_approval($user_id)
{

    $new_status = 'awaiting_admin_review';
    update_user_meta($user_id, 'account_status', $new_status);
    $user_info = get_userdata($user_id);
    $admin_profile_url = site_url("wp-admin/user-edit.php?user_id={$user_id}");
    $display_name = $user_info->display_name;
    $site_name = get_bloginfo('name');

    // Define the fields you want to include
    $desired_fields = ['phone_number', 'user_email', 'anrede', 'company_name', 'first_name', 'last_name'];

    // Prepare the submitted_registration string
    $submitted_registration = "";
    foreach ($desired_fields as $key) {
        if (isset($user_info->$key)) {  // see if the user has this field
            $submitted_registration .= "{$key}: " . $user_info->$key . "<br>";
        }
    }


    $to = get_option('admin_email');
    UM()->mail()->send(
        $to,
        'admin_account_mail_verification',
        array(
            'tags' => array(
                '{admin}',
                '{email}',
                '{username}',
                '{display_name}',
                '{site_name}',
                '{admin_profile_url}',
                '{account_data}'
            ),
            'tags_replace' => array(
                'Admin',
                $user_info->user_email,
                $user_info->user_login,
                $display_name,
                $site_name,
                $admin_profile_url,
                $submitted_registration
            )
        )
    );
    /* ToDo add it to an adminpage to change page dynamically */
    $redirect_url = home_url('/account-confirmation');
    exit(wp_redirect($redirect_url));
}





add_filter('um_email_templates', 'add_custom_um_email_templates', 10, 1);
/**
 * Adds a custom email template to the given array of templates.
 *
 * @param array $templates The array of existing email templates.
 * @return array The updated array of templates with the custom template added.
 */
function add_custom_um_email_templates($templates)
{
    $templates[] = 'custom-template';
    return $templates;
}


if (!function_exists('um_email_locate_template')) {
    /**
     * Locate a template and return the path for inclusion.
     */
    function um_email_locate_template($template_name)
    {
        $blog_id = is_multisite() ? '/' . get_current_blog_id() : '';

        $template = locate_template(
            array(
                trailingslashit('ultimate-member/email' . $blog_id) . $template_name . '.php',
                trailingslashit('ultimate-member/email') . $template_name . '.php',
            )
        );

        if (!$template) {
            $template = wp_normalize_path(STYLESHEETPATH . '/ultimate-member/email/' . $template_name . '.php');
        }

        return apply_filters('um_locate_email_template', $template, $template_name);
    }
}

/**
 * Add custom email template "Profile Completeness - Profile is complete".
 *
 * @param  array $emails Emails list.
 * @return array
 */
function um_email_notification_profile_is_complete($emails)
{


    $review_mail = $emails["notification_review"];



    // New email templates.
    $custom_emails = array(
        'admin_account_mail_verification' => array(
            'key' => 'admin_account_mail_verification',
            'title' => __('admin_account_mail_verification', 'ultimate-member'),
            'description' => __('Whether to send an email to an admin when a member\'s profile is complete', 'ultimate-member'),
            'recipient' => 'admin',
            'default_active' => true,
            'subject' => $review_mail['subject'],
            'body' => $review_mail['body'],
        ),
    );

    foreach ($custom_emails as $slug => $custom_email) {
        // Default settings.
        $email_option_on = UM()->options()->get($slug . '_on');
        $email_option_sub = UM()->options()->get($slug . '_sub');
        if ('' === $email_option_on) {
            $email_on = empty($custom_email['default_active']) ? 0 : 1;
            UM()->options()->update($slug . '_on', $email_on);
        }
        if ('' === $email_option_sub) {
            $email_sub = empty($custom_email['subject']) ? $slug : $custom_email['subject'];
            UM()->options()->update($slug . '_sub', $email_sub);
        }

        // Template file.
        $located = um_email_locate_template($slug);
        if (!file_exists($located)) {
            wp_mkdir_p(dirname($located));
            file_put_contents($located, $custom_email['body']);
        }

        $emails[$slug] = $custom_email;
    }

    return $emails;
}
add_filter('um_email_notifications', 'um_email_notification_profile_is_complete');




add_action('woocommerce_account_myprofil_endpoint', 'woocommerce_account_profil_endpoint');


add_filter('um_profile_tabs', 'umember_custom_tab', 1000);
/**
 * Removes the 'posts' and 'comments' tabs from the given array of tabs.
 *
 * @param array $tabs The array of tabs to modify.
 * @return array The modified array of tabs.
 */
function umember_custom_tab($tabs)
{

    unset($tabs['posts']);
    unset($tabs['comments']);
    return $tabs;
}
/**
 * Render tab content
 *
 * @param array $args
 */



function new_woocommerce_account_menu_order($items, $endpoints)
{

    $menuOrder = array(
        'dashboard' => __('Dashboard', 'woocommerce'),
        'edit-address' => __('Rechungsadresse', 'woocommerce'),
        'myprofil' => __('Mein Unternehmensprofil', 'woocommerce'), // replace 'unternehmensprofil' with the actual endpoint
        'myjobs' => __('Meine Anzeigen', 'woocommerce'), // replace 'anzeigen' with the actual endpoint
        //'orders' => __('Bestellungen', 'woocommerce'),
        'edit-account' => __('Kontodetails', 'woocommerce'),
        'customer-logout' => __('Abmelden', 'woocommerce'),
    );
    return $menuOrder;
}
add_filter('woocommerce_account_menu_items', 'new_woocommerce_account_menu_order', 2, 10);
/**
 * Retrieves the post title for a given post ID.
 *
 * @param array $atts Optional. An array of attributes to customize the function.
 * @return string The post title.
 */
function custom_cf7_get_post_title($atts)
{
    if (isset($_GET['post_id'])) {
        $post_id = $_GET['post_id'];
    }
    $atts = shortcode_atts(
        array(
            'post_id' => '',
        ),
        $atts,
        'get_title'
    );

    return get_the_title($post_id);

}
add_shortcode('get_title', 'custom_cf7_get_post_title');
/**
 * Retrieves a custom field value for a specific post.
 *
 * @param array $atts An array of attributes for the shortcode.
 * @return string The value of the custom field.
 */
function custom_cf7_get_post_var($atts)
{
    if (isset($_GET['post_id'])) {
        $post_id = $_GET['post_id'];
    }
    $atts = shortcode_atts(
        array(
            'key' => '',
            'post_id' => '',
        ),
        $atts,
        'get_post_var'
    );
    return get_post_meta($post_id, $atts['key'], true);
}
add_shortcode('get_get_var', 'custom_cf7_get_post_var');
/**
 * Modifies the job_company field in a Contact Form 7 form.
 *
 * @param array $tag The tag being modified.
 * @param array $unused Unused parameter.
 * @return array The modified tag.
 */
function modify_cf7_field($tag, $unused)
{
    if ($tag['name'] !== 'job_company')
        return $tag;

    if (is_user_logged_in()) {
        $user_id = get_current_user_id();
        $args = array(
            'posts_per_page' => -1,
            'post_type' => 'company',

            'meta_query' => array(
                array(
                    'key' => 'user_id',
                    'value' => $user_id,
                    'compare' => '=',
                )
            )
        );
        $companies = get_posts($args);

        if ($companies) {

            $tag['type'] = 'select';
            $tag['raw_values'] = [];
            $tag['raw_labels'] = [];

            foreach ($companies as $company) {
                $tag['options'][] = 'value:' . $company->post_title;
                $tag['raw_values'][] = $company->post_title;
                $tag['labels'][] = $company->post_title;
                $tag['raw_labels'][] = $company->post_title;
            }
        }
        return $tag;
    }

    return $tag;
}
add_filter('wpcf7_form_tag', 'modify_cf7_field', 10, 2);

add_filter('wpcf7_form_elements', function ($content) {
    if (is_user_logged_in()) {
        $user_id = get_current_user_id();
        $args = array(
            'posts_per_page' => -1,
            'post_type' => 'company',

            'meta_query' => array(
                array(
                    'key' => 'user_id',
                    'value' => $user_id,
                    'compare' => '=',
                )
            )
        );
        $companies = get_posts($args);

        if (!empty($companies)) {
            $options = "";
            foreach ($companies as $company) {
                $options .= '<option data-value="' . $company->ID . '" value="' . $company->post_title . '">' . $company->post_title . '</option>';
            }
            $options .= '<option data-value="" value="new">Freies Textfeld</option>';
            $pattern = '/<input size="40" maxlength="400" class="wpcf7-form-control wpcf7-select wpcf7-text uk-input" aria-invalid="false" placeholder="Arbeitgeber" value="" type="text" name="job_company"[^>]*>/';
            $replacement = '<select name="job_company" class="wpcf7-form-control wpcf7-select wpcf7-validates-as-required uk-select">' . $options . '</select>';
            $replacement .= '<input type="hidden" name="company_id" value="' . $companies[0]->ID . '">';
            $replacement .= '<div id="custom-input" class="uk-margin" style="display: none;"><input name="job_company" class="uk-input" value="' . $companies[0]->post_title . '" type="text" placeholder="Arbeitsgeber"></div>';

            $content = preg_replace($pattern, $replacement, $content);
        }
    }
    return $content;
});



