<?php
/**
 * Class Class_cf7_Helper
 * @package LeadGenerator
 *
 * @since 0.2.1
 */
class Class_cf7_Helper
{
    /**
     * Constructor function for the class.
     * 
     * This function initializes the class by calling the build_tags() method and adding necessary actions and shortcodes.
     * 
     * @since 1.0.0
     * @access public
     * @return void
     */
    function __construct()
    {
        $this->build_tags();
        add_action('wpcf7_before_send_mail', [$this, 'action_wpcf7_before_send_mail'], 10, 1);
        add_action('wpcf7_init', [$this, 'custom_tags']);
        add_shortcode('accept_job', [$this, 'accept_uuid']);


    }
    /**
     * Retrieves all ACF field groups for a given post type.
     *
     * @param string $cpt The post type to retrieve field groups for.
     * @return array An array containing all ACF field groups for the given post type.
     */
    public function get_acf_field_groups_by_cpt($cpt)
    {

        $result = array(
            'fields' => []
        );
        $acf_field_groups = acf_get_field_groups();
        foreach ($acf_field_groups as $acf_field_group) {

            foreach ($acf_field_group['location'] as $group_locations) {
                foreach ($group_locations as $rule) {

                    if ($rule['param'] == 'post_type' && $rule['operator'] == '==' && $rule['value'] == $cpt) {

                        $result['fields'] = array_merge($result['fields'], acf_get_fields($acf_field_group));


                    }

                }

            }

        }

        return $result;
    }

    //$groups = acf_get_field_groups(array('post_type' => 'your-post-type'));
    /**
     * Builds tags for the Contact Form 7 plugin based on the Advanced Custom Fields (ACF) field groups for the 'job' custom post type.
     *
     * @return void
     */
    public function build_tags()
    {
        $fields = $this->get_acf_field_groups_by_cpt('job')['fields'];
        foreach ($fields as $field) {

            wpcf7_add_form_tag(
                'lead_' . $field['name'],
                array($this, 'wpcf7_current_url_form_tag_handler'),
                array(
                    'name-attr' => true
                )
            );
        }
        wpcf7_add_form_tag(
            'acf',
            array($this, 'acf_form_tag_handler'),
            array(
                'name-attr' => true
            )
        );

    }
    /**
     * Retrieves the ACF key for a given field name.
     *
     * @param string $field_name The name of the ACF field.
     * @return string|bool The ACF key if found, or false if not found.
     */
    public function get_acf_key($field_name)
    {
        $field = get_field_object($field_name);

        if (isset($field['key'])) {
            return $field['key'];
        } else {
            return false;
        }
    }


    // Parse the shortcode in the frontend
    /**
     * Handles the current URL form tag for the Contact Form 7 plugin.
     *
     * @param array $tag The form tag to be handled.
     * @return string The processed form tag.
     */
    public function wpcf7_current_url_form_tag_handler($tag)
    {
        global $wp;
        $name = str_replace('lead_', '', $tag['type']);
        $field = acf_get_field($name);

        return $this->switch_types($field['type'], $field);

    }
    /**
     * This function takes in a type and a field and returns the appropriate HTML input or select element based on the type.
     *
     * @param string $type The type of input element to be returned.
     * @param array $field An array containing the necessary information for the input or select element.
     * @return string The HTML input or select element.
     */
    function switch_types($type, $field)
    {


        switch ($type) {
            case 'text':
                return '<input type="' . $field['type'] . '" name="' . $field['name'] . '" value="" />';
                break;
            case 'number':
                return '<input type="' . $field['type'] . '" name="' . $field['name'] . '" value="" />';
                break;
            case 'email':
                return '<input type="' . $field['type'] . '" name="' . $field['name'] . '" value="" />';
                break;
            case 'url':
                return '<input type="' . $field['type'] . '" name="' . $field['name'] . '" value="" />';
                break;
            case 'radio':
                return '<input type="' . $field['type'] . '" name="' . $field['name'] . '" value="" />';
                break;
            case 'select':
                return '<select name="' . $field['name'] . '" id="' . $field['name'] . '">
           
              </select>';
                break;
        }
        ;

    }
    /**
     * Function to handle bulk image uploads for a form field.
     *
     * @param WPCF7_FormTag $tag The form tag to be processed.
     * @return string The compiled template for the form field.
     */
    function bulk_image_upload_handler($tag)
    {
        $tag = new WPCF7_FormTag($tag);
        $template = apply_filters('wp_leadgen_template_path', '/form/fields/galleries.twig');

        return Timber::compile($template, ["tag" => $tag]);
    }
    /**
     * Adds custom form tags for use in Contact Form 7.
     *
     * @return void
     */
    function custom_tags()
    {



        wpcf7_add_form_tag('multi_files', [$this, 'bulk_image_upload_handler']);
        wpcf7_add_form_tag('uuid', [$this, 'custom_uuid_tag_handler']); // "clock" is the type of the form-tag
        wpcf7_add_form_tag('auto_geo_field', [$this, 'auto_geo_field'], ['name-attr' => true]); // "clock" is the type of the form-tag
        wpcf7_add_form_tag('user_id', [$this, 'user_id_field']); // "clock" is the type of the form-tag
        wpcf7_add_form_tag('company_id', [$this, 'company_id_field']); // "clock" is the type of the form-tag
        wpcf7_add_form_tag('user_companies', [$this, 'user_companies_field']); // "clock" is the type of the form-tag
        wpcf7_add_form_tag('country_dropdown', [$this, 'render_country_dropdown']);
        wpcf7_add_form_tag(
            'acf',
            array($this, 'acf_form_tag_handler'),
            array(
                'name-attr' => true
            )
        );
        wpcf7_add_form_tag(
            'file_preview',
            [$this, 'custom_handler_for_file_field'],
            ['name-attr' => true],
        );
        wpcf7_add_form_tag(
            'gallery',
            [$this, 'custom_gallery_preview_handler_for_file_field'],
            ['name-attr' => true],
        );
        wpcf7_add_form_tag(
            'post_taxonomy',
            [$this, 'custom_post_taxonomy_form_tag_handler'],
            array(
                'name-attr' => true,
                'autocomplete' => true,
                'class' => true
            )
        );
    }
    /**
     * ACF Form Tag Handler function
     *
     * Handles the rendering of ACF form tags and returns the appropriate template based on the field type.
     *
     * @param array $tag The ACF form tag to be rendered.
     * @return string The rendered template for the ACF form tag.
     */
    public function acf_form_tag_handler($tag)
    {

        $field = get_field_object($tag['raw_name']);

        if ($field['type'] == 'checkbox') {
            $template = apply_filters('wp_leadgen_template_path', 'form/fields/checkbox.twig');
            //required:true 
            $post_ID = $this->get_post_ID_from_url();
            $field["post_id"] = $post_ID;
            $field["value"] = get_post_meta($field['post_id'], $field['name'], True);


            return Timber::compile($template, $field);
        }

        if ($field['type'] == 'select') {
            $post_ID = $this->get_post_ID_from_url();
            $field["post_id"] = $post_ID;
            $field["value"] = get_post_meta($field['post_id'], $field['name'], True);

            $template = apply_filters('wp_leadgen_template_path', 'form/fields/select.twig');
            return Timber::compile($template, $field);
        }

    }
    /**
     * Renders a custom gallery preview for a file field.
     *
     * @param string $tag The tag for the file field.
     * @return string The rendered gallery preview.
     */
    public function custom_gallery_preview_handler_for_file_field($tag)
    {
        $post_id = $this->get_post_ID_from_url();


        $impressionen = get_field('impressionen', $post_id);

        $template = apply_filters('wp_leadgen_template_path', '/form/fields/galleries.twig');
        $data = [
            'tag' => $tag,
            "images" => $impressionen
        ];

        $result = Timber::compile($template, $data);

        return $result;
    }
    /**
     * Custom handler for file field.
     *
     * @param $tag The tag object.
     * @return string The compiled template.
     */
    public function custom_handler_for_file_field($tag)
    {
        $result = '';


    

        if ($tag->name === 'company_signet') {
            $post_id = $this->get_post_ID_from_url();


            if (has_post_thumbnail($post_id)) {
                $imageSrc = get_the_post_thumbnail_url($post_id);
                $attachment_id = get_post_thumbnail_id($post_id);
            }

            $template = apply_filters('wp_leadgen_template_path', '/form/fields/file.twig');

            $data = [

                'tag' => $tag,
                "imageSrc" => $imageSrc,
                "attachment_id" => $attachment_id,
            ];

            $result = Timber::compile($template, $data);

            return $result;
        } else {
            $post_id = $this->get_post_ID_from_url();
            $image = get_field($tag->name, $post_id);

            $template = apply_filters('wp_leadgen_template_path', '/form/fields/file.twig');
            if (isset($image["url"])) {
                $imageSrc = $image["url"];
            } else {
                $imageSrc = "";
            }

            $data = [

                'tag' => $tag,
                "imageSrc" => $imageSrc,
                "attachment_id" => $image['ID'],
            ];


            $result = Timber::compile($template, $data);

            return $result;
        }

    }
    /**
     * Generates a form field for auto-geolocation.
     *
     * @param string $tag The form tag.
     * @return string The generated form field.
     */
    function auto_geo_field($tag)
    {
        $tag = new WPCF7_FormTag($tag);
        $class = wpcf7_form_controls_class($tag->type); // Getting form control class


        // Creating hidden fields
        $fields = ['city', 'country', 'zip', 'latitude', 'longitude'];
        $hiddenFields = '';
        foreach ($fields as $field) {

            $hiddenFields .= '<input type="hidden" class="' . $class . '" name="' . $field . '" id="' . $field . '" />';

        }

        // Creating address input field
/* 
        <span class="wpcf7-form-control-wrap" data-name="your-name">
    <input class="wpcf7-form-control wpcf7-text" type="text" name="your-name" />
*/
        $addressField = '<span class="wpcf7-form-control-wrap" data-name="' . $tag->type . '">';
        $addressField .= '<input type="text" class="wpcf7-form-control uk-input city_placeholder" id="searchInput" name="adress" placeholder="Einsatzort" required="required" />';
        $addressField .= '<span role="alert" class="wpcf7-not-valid-tip" style="display: none;"></span>';
        $addressField .= '</span>';

        $output = $hiddenFields . $addressField;
        return $output;
    }
    /**
     * Retrieves the post ID from the URL if it exists and is not empty.
     *
     * @return int The post ID from the URL, or 0 if it does not exist or is empty.
     */
    private function get_post_ID_from_url()
    {

        if (isset($_GET['post_id']) && $_GET['post_id']) {
            return intval($_GET['post_id']);
        }
    }
    /**
     * A function that handles the custom post taxonomy form tag.
     *
     * @param array $tag The form tag array.
     * @return string The compiled template for the form tag.
     */
    function custom_post_taxonomy_form_tag_handler($tag)
    {

        $tag = new WPCF7_FormTag($tag);
        $title = $tag->get_option('title', '');
        $id = $tag->get_option('id', '');
        $required = $tag->get_option('required', False);


        $terms = get_terms(
            array(
                'taxonomy' => $tag['name'],
                'hide_empty' => false,
            )
        );
        $value = $this->get_post_ID_from_url();
        if ($value) {
            $value = get_the_terms($value, $tag['name']);
            $value = wp_list_pluck($value, 'name');
        }


        $template = apply_filters('wp_leadgen_template_path', '/form/terms.twig');

       
        $data = [
            'terms' => $terms,
            'id' => $id[0],
            'tag' => $tag,
           // 'title' => $title[0],
            'required' => $required ? $required[0] : '',
            'value' => $value,
        ];


        return Timber::compile($template, $data);

    }
    /**
     * Retrieves the list of companies associated with the current user and displays them as input fields or checkboxes.
     *
     * @param $tag The tag to be used for the input fields or checkboxes.
     * @return string The HTML output of the input fields or checkboxes.
     */
    function user_companies_field($tag)
    {

        $user = get_current_user_id();


        if (!isset($user)) {
            return;
        } else {
            $output = '';
            $user = get_userdata($user);
            $companies = Account::get_companies($user->ID);
            if (count($companies) == 1) {
                foreach ($companies as $company) {
                    $output .= '<input id="' . $company->ID . '" name="company_id" value="' . $company->ID . '" type="hidden"/>';

                }
            }
            if (count($companies) > 1) {
                foreach ($companies as $company) {
                    $output .= '<input id="' . $company->ID . '" name="company_id" value="' . $company->ID . '" type="checkbox"/>';
                    $output .= '<label for="' . $company->ID . '">' . get_the_title($company->ID) . '</label>';
                }
            }
            return $output;


        }
    }
    /**
     * Generates a hidden input field for the current user's ID.
     *
     * @param string $tag The HTML tag for the input field.
     * @return string The generated HTML for the input field.
     */
    function user_id_field($tag)
    {



        // Execute your code for the customer
        $user_id = get_current_user_id();

        $output = '<input type="hidden" value="' . $user_id . '" name="user_id" id="user_id"  />';





        return $output;

    }
    /**
     * Retrieves the company ID field for a given tag.
     *
     * @param string $tag The tag to retrieve the company ID field for.
     * @return string The HTML output for the company ID field.
     */
    function company_id_field($tag)
    {



        // Execute your code for the customer
        $company_id = $_GET['post_id'];

        $output = '<input type="hidden" value="' . $company_id . '" name="company_id" id="company_id"  />';





        return $output;

    }
    /**
     * Generates a custom UUID tag handler.
     *
     * @param string $tag The tag to be handled.
     * @return string The generated UUID.
     */
    function custom_uuid_tag_handler($tag)
    {

        return $this->generate_uuid();
    }
    /**
     * Creates a new attachment for a given file and post ID.
     *
     * @param string $filename The name of the file to be attached.
     * @param int $id The ID of the post to attach the file to.
     * @return int The ID of the newly created attachment.
     */
    public function nmr_create_attachment($filename, $id)
    {
        // Check the type of file. We'll use this as the 'post_mime_type'.
        $filetype = wp_check_filetype(basename($filename), null);

        // Get the path to the upload directory.
        $wp_upload_dir = wp_upload_dir();

        $attachFileName = $wp_upload_dir['path'] . '/' . basename($filename);
        $attachFileName = apply_filters('nmr_create_attachment_file_name', $attachFileName);
        copy($filename, $attachFileName);
        // Prepare an array of post data for the attachment.
        $attachment = array(
            'guid' => $attachFileName,
            'post_mime_type' => $filetype['type'],
            'post_title' => preg_replace('/\.[^.]+$/', '', basename($filename)),
            'post_content' => '',
            'post_status' => 'inherit',
            'post_parent' => $id,
        );
        // Notify subscribers with attachment data
        $attachment = apply_filters('nmr_before_insert_attachment', $attachment);
        // Insert the attachment.
        $attach_id = wp_insert_attachment($attachment, $attachFileName);

        // Make sure that this file is included, as wp_generate_attachment_metadata() depends on it.
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        // Required for audio attachments
        require_once(ABSPATH . 'wp-admin/includes/media.php');

        // Generate the metadata for the attachment, and update the database record.
        $attach_data = wp_generate_attachment_metadata($attach_id, $attachFileName);
        wp_update_attachment_metadata($attach_id, $attach_data);
        do_action('nmr_create_attachment_id_generated', $attach_id);
        return $attach_id;
    }
    /**
     * Accepts a job post and updates its status to pending.
     *
     * @param array $data The data containing the job accept key.
     * @return void
     */
    public function action_accept_job_post($data)
    {
        if (isset($data['job_accept'])) {

            $query = new WP_Query(
                array(
                    //    'post_status' => 'unconfirmed',
                    'post_type' => 'job',
                    'meta_key' => 'uuid',
                    'meta_value' => $data['job_accept']
                )
            );

            if ($query->posts) {

                // Update post
                $job = array();
                $job['ID'] = $query->posts[0]->ID;
                $job['post_status'] = 'pending';
                // Update the post into the database
  

                wp_update_post($job);
            }
        }


    }
    /**
     * Validates company data to ensure that the company ID and user ID match.
     * 
     * @param array $data The data to be validated.
     * @return bool True if the data is valid, false otherwise.
     */
    private function validate_company_data($data)
    {


        if (isset($data['company_id'])) {
            $company_id = $data['company_id'];
            $user_id = get_post_meta($company_id, 'user_id', true);
            if ($user_id != $data['user_id']) {
                return False;
            }
            return True;
        }
        return True;
    }
    /**
     * Action for company form submission.
     * 
     * @param array $data The data submitted from the form.
     * @param array $submission The submission data.
     * @return array The submission data.
     */
    public function action_company_form($data, $submission)
    {
        if (isset($data['company_id']) && $data['company_id'] != 'new') {
            if (!$this->validate_company_data($data) || !is_array($data)) {

                return 'not valide';
            }

            $args = array(
                'ID' => intval($data['company_id']),
                'post_title' => $data['post_title'],
                'post_content' => $data['post_content'],
                'post_name' => $data['post_title'],
                'post_type' => 'company',
                'post_status' => 'publish',
                'meta_input' => $data,
            );

            $id = wp_insert_post($args);


        } elseif ($data['company_id'] == 'new') {
            $args = array(
                'post_title' => $data['post_title'],
                'post_status' => 'publish',
                'post_content' => $data['post_content'],
                'post_name' => $data['post_title'],
                'post_type' => 'company',
                'meta_input' => $data,
            );
            $id = wp_insert_post($args);

        }




        $ids = $this->import_gallery($data);

        update_field('impressionen', $ids, $id);


        if ($data['jobcategory_terms']) {
            $this->set_terms_for_job($id, $data);
        }
        $address = GeoAdress::validate_adress($data);
        GeoAdress::set_adress($address, $id);


        $this->set_uploads($id, $submission);

        return $submission;


    }
    /**
     * Imports a gallery of images into the WordPress media library.
     *
     * @param array $data An array of data containing the images to be imported.
     * @return array An array of attachment IDs for the imported images.
     */
    public function import_gallery($data)
    {

        require_once(ABSPATH . 'wp-admin/includes/media.php');
        require_once(ABSPATH . 'wp-admin/includes/file.php');
        require_once(ABSPATH . 'wp-admin/includes/image.php');

        $attachment_ids = array();

        foreach ($data as $key => $value) {
            // If the key starts with 'gallery_images_', we have a base64 string
            if (strpos($key, 'gallery_images_') !== false) {
                $index = str_replace('gallery_images_', '', $key);
                $attach_id = $this->create_attachment_from_base64($value, 0, $index);
                if ($attach_id) {
                    $attachment_ids[] = $attach_id;
                }
            } else if ($key === 'impressionen_gallery') { // Existing images
                $attachment_ids = array_merge($attachment_ids, $value);
            }
        }

        return $attachment_ids;
    }
    /**
     * Creates an attachment from a base64 encoded string and attaches it to a post.
     *
     * @param string $base64_string The base64 encoded string of the image.
     * @param int $post_id The ID of the post to attach the image to.
     * @param int $index The index of the image in the post.
     * @return int|false The ID of the created attachment or false if an error occurred.
     */
    private function create_attachment_from_base64($base64_string, $post_id, $index)
    {
        $upload_dir = wp_upload_dir();
        $image_data = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $base64_string));

        $f = finfo_open();
        $mime_type = finfo_buffer($f, $image_data, FILEINFO_MIME_TYPE);

        //Allow only jpg, jpeg, gif and png file types
        if (!in_array($mime_type, ['image/jpeg', 'image/gif', 'image/png'])) {
            return false;
        }

        $extension = ltrim(strrchr($mime_type, '/'), '/');
        $file = $upload_dir['path'] . '/impressionen_' . $index . '.' . $extension;

        file_put_contents($file, $image_data);

        $wp_filetype = wp_check_filetype($file);

        $attachment = array(
            'guid' => $upload_dir['url'] . '/' . basename($file),
            'post_mime_type' => $wp_filetype['type'],
            'post_title' => preg_replace('/\.[^.]+$/', '', basename($file)),
            'post_content' => '',
            'post_status' => 'inherit'
        );

        $attach_id = wp_insert_attachment($attachment, $file, $post_id);
        $attach_data = wp_generate_attachment_metadata($attach_id, $file);
        wp_update_attachment_metadata($attach_id, $attach_data);

        return $attach_id;
    }
    /**
     * This function is executed before sending an email through the Contact Form 7 plugin.
     * It handles the submission data and performs various actions based on the form settings.
     *
     * @param $contact_form The Contact Form 7 object.
     * @return $contact_form The updated Contact Form 7 object.
     */
    public function action_wpcf7_before_send_mail($contact_form)
    {

        $submission = WPCF7_Submission::get_instance();
        if ($submission) {
            $data = $submission->get_posted_data();


            if (get_post_meta($contact_form->id, 'leadgen-accept-post', true) == 'on') {
                $this->action_accept_job_post($data);
            }
            if (get_post_meta($contact_form->id, 'leadgen-company_form', true) == 'on') {
                $submission = $this->action_company_form($data, $submission);

            }

            if (get_post_meta($contact_form->id, 'leadgen-on', true) == 'on') {

                if (isset($data['use_invoice_address_checkbox'])) {
                    if ($data['use_invoice_address_checkbox'][0] == 'Nutze Rechnungsaddresse') {
                        $job_data = $this->copy_customer_data_to_job_data($data);
                        $data = array_merge(
                            $data,
                            $job_data
                        );

                    }
                }

                if (isset($data['country'])) {
                    $data['country'] = $data['customer_country'];
                }
                if (isset($data['company'])) {
                    $data['company'] = $data['customer_company'];
                }

        

                $args = array(
                    'post_title' => $data['post_title'],
                    'post_content' => $data['post_content'],
                    'post_name' => $data['uuid'],
                    'post_type' => 'job',
                    'post_status' => 'unconfirmed',
                    'meta_input' => $data,
                );

                $id = wp_insert_post($args);


                $address = GeoAdress::validate_adress($data);
                GeoAdress::set_adress($address, $id);


                $this->set_terms_for_job($id, $data);
                $this->set_uploads($id, $submission);
                $post = get_post($id);


                $LGenPost = new LGenPost();
                $job = $LGenPost->setup_job($post);


                if (empty($data['longitude']) && empty($data['latitude'])) {
                    if ($job->street && $job->city && $job->zip) {
                        $geo_data = Nominatim::geocode($job->build_address());
                        update_post_meta($id, 'longitude', $geo_data['lon']);
                        update_post_meta($id, 'latitude', $geo_data['lat']);
                    }
                }

                if (isset($data['product'])) {
                    update_post_meta($id, 'product_id', $data['product'][0]);
                    $order_id = Class_woocommerce_Helper::create_order($data['product'][0], $id);


                    update_post_meta($id, 'product_id', $data['product'][0]);
                    update_post_meta($id, 'order_id', $order_id);

                    if (get_field('premium', $data['product'][0], true)) {
                        update_post_meta($id, 'premium', 1);
                    }
                    $customer_data = $this->parse_customer_data($data);
                    Class_woocommerce_Helper::set_billing_address($order_id, $customer_data);

                }

                $mail = $contact_form->prop('mail'); // returns array

                $uuid_link = $this->generate_uuid_link($data['uuid']);


                $mail['body'] = str_replace("[uuid]", $uuid_link, $mail['body']);

                $contact_form->set_properties(array('mail' => $mail));
       
      

            }
        }

        return $contact_form;
    }
    /**
     * Sets the uploaded files for a specific submission.
     *
     * @param int $id The ID of the submission.
     * @param object $submission The submission object.
     */
    function set_uploads($id, $submission)
    {
        $uploaded_files = $submission->uploaded_files();
    

          
  

        if (isset($uploaded_files)) {


            $files = $submission->uploaded_files();
    
            $data = $submission->get_posted_data();
     
            foreach ($uploaded_files as $fieldName => $filepath) {

                //cf7 5.4
                if (is_array($filepath)) {
        
                    foreach ($filepath as $key => $value) {
                        $attachment_id = $this->nmr_create_attachment($value, $id);
                        $data[$fieldName] = $attachment_id;
                        if ($fieldName === 'company_signet') {
                            $post = get_post($id);
                            set_post_thumbnail($post, $attachment_id);

                        }
                        if ($fieldName === 'company_background_image') {
               
                            update_field('company_background_image', $attachment_id, $id);
                        }
                        if ($fieldName === 'job_banner') {
                       
                            update_field('job_banner', $attachment_id, $id);

                        }
                         


                        if ($fieldName === 'company_job_description') {
                            $post = get_post($id);
                            update_field('company_job_description', $attachment_id, $id);

                        }
                        if ($fieldName === 'impressionen') {
                            array_push($gallery_ids, $attachment_id);


                        }
                    }
                } else {

                    $attachment_id = $this->nmr_create_attachment($filepath, $id);

                    if ($fieldName === 'company_job_description') {
                        $post = get_post($id);
                        update_field('company_job_description', $attachment_id, $id);

                    }

                }

            

            }


        }
        

        require_once(ABSPATH . "wp-admin" . '/includes/image.php');
        require_once(ABSPATH . "wp-admin" . '/includes/file.php');
        require_once(ABSPATH . "wp-admin" . '/includes/media.php');

        $gallery_ids = [];

        foreach ($files as $input_field_name => $file_path) {
            if (strpos($input_field_name, 'impressionen_') === 0) { // if starts with impressionen_
                $index = explode("impressionen_", $input_field_name)[1];
                $file_content = file_get_contents($file_path);
                $attach_id = create_attachment_from_base64($file_content, $id /* replace with your post id */ , $index);
                $gallery_ids[] = $attach_id;
            }
        }

        // After looping through all files, update the ACF `gallery` field
        if (!empty($gallery_ids)) {
            update_field('impressionen', $gallery_ids, $id);
        }
       
    }
    /**
     * Creates and returns geographic data.
     *
     * @param mixed $data The data to be used for creating the geographic data.
     * @return mixed The created geographic data.
     */
    function create_geo_data($data)
    {

        return $data;
    }
    /**
     * Returns an array of keys from the given data array that end with the string "_terms".
     *
     * @param array $data The data array to search through.
     * @return array An array of keys that end with the string "_terms".
     */
    private function get_keys_ending_with_terms($data)
    {
        $keys = array_keys($data);
        $result = [];
        foreach ($keys as $key) {
            if (strpos(strrev($key), strrev("_terms")) === 0) {
                $result[] = $key;
            }
        }
        return $result;
    }
    /**
     * Sets the terms for a job post.
     *
     * @param int $post_id The ID of the job post.
     * @param array $data The data containing the terms to be set.
     */
    function set_terms_for_job($post_id, $data)
    {
        $terms = $this->get_keys_ending_with_terms($data);

        foreach ($terms as $term_data) {
            $term = str_replace('_terms', '', $term_data);
            foreach ($data[$term_data] as $term_id) {

                wp_set_object_terms($post_id, intval($term_id), $term, true);
            }
        }

    }
    /**
     * Parses customer data and returns an array of customer information.
     *
     * @param array $data The data to be parsed.
     * @return array An array of customer information.
     */
    function parse_customer_data($data)
    {
        $customer_data = [];
        foreach ($data as $key => $field) {
            if (str_starts_with($key, 'customer_')) {
                $new_key = str_replace("customer_", "", $key);
                $customer_data[$new_key] = $field;
            }
        }
        return $customer_data;
    }
    /**
     * Copies customer data from an array to a new array with modified keys.
     *
     * @param array $data The array containing customer data.
     * @return array The new array with modified keys.
     */
    function copy_customer_data_to_job_data($data)
    {
        $customer_data = [];
        foreach ($data as $key => $field) {
            if (
                str_starts_with($key, 'customer_') && in_array($key, [
                    'customer_street',
                    'customer_street2',
                    'customer_zip',
                    'customer_city'
                ])
            ) {
                $new_key = str_replace("customer_", "", $key);
                $customer_data[$new_key] = $field;
            }
        }
        return $customer_data;
    }
    /**
     * Generates a unique identifier (UUID) and returns it as a hidden input field.
     *
     * @return string The generated UUID as a hidden input field.
     */
    function generate_uuid()
    {

        $uniqid = uniqid();
        return '<input type="hidden" name="uuid" value="' . $uniqid . '">';
    }
    /**
     * Generates a unique URL for confirming a job acceptance.
     *
     * @param string $uniqid The unique identifier for the job.
     * @return string The generated URL for confirming the job acceptance.
     */
    function generate_uuid_link($uniqid)
    {
        $base_url = get_site_url();
        return $base_url . '/confirm?job_accept=' . $uniqid;
    }
    /**
     * Accepts a UUID and updates the corresponding job post to 'pending' status.
     *
     * @param array $atts Optional. Shortcode attributes. Default empty array.
     * @return void
     */
    function accept_uuid($atts = array(), $content = null, $tag = '')
    {
        $atts = shortcode_atts(
            array(
                'redirect' => 679
            ),
            $atts,
            $tag
        );

        if (isset($_GET['job_accept']) && $_GET['job_accept']) {
            $query = new WP_Query(

                array(
                    'post_status' => ['any'],
                    'post_type' => 'job',
                    'meta_key' => 'uuid',
                    'meta_value' => $_GET['job_accept']
                )
            );

            if ($query->posts) {

                // Update post
                $job = array();
                $job['ID'] = $query->posts[0]->ID;
                $job['post_status'] = 'pending';
                // Update the post into the database
                wp_update_post($job);

                do_action('leadgen_job_confirmed', $job);
                wp_redirect(get_the_permalink($atts['redirect']), 301);


            }



        }
    }
    /**
     * Renders a dropdown menu for selecting a country.
     *
     * @param string $tag The tag used for the dropdown menu.
     * @return string The rendered HTML for the dropdown menu.
     */
    function render_country_dropdown($tag)
    {
        // Get WooCommerce countries
        $countries = WC()->countries->get_countries();
        $template = apply_filters('wp_leadgen_template_path', 'form/fields/countries.twig');
        $post_ID = $this->get_post_ID_from_url();
        return Timber::compile($template, [
            'countries' => $countries,
            'value' => get_post_meta($post_ID, 'country', true)
        ]);



    }
}








