<?php

/**
 * Class LGenQuery
 * @package LeadGenerator
 *
 * @since 0.2.1
 */
class LGenQuery
{
    public $args;
    public $meta_query;
    /**
     * Constructor for the class.
     * Sets the default arguments for the query and the meta query.
     *
     * @return void
     */
    function __construct()
    {
        $this->args = [
            'post_type' => 'job',
            'posts_per_page' => 10,
            'post_status' => 'publish',
        ];
        $this->meta_query = [];
    }
    /**
     * Retrieves all jobs that are past their deadline and ready to be deleted.
     *
     * @return int The number of jobs to be deleted.
     */
    public function get_all_jobs_to_delete()
    {
        $args = array(
            'fields' => 'ids',    // This line was added
            'meta_query' => array(
                array(
                    'key' => 'job_ad_deadline', //replace with your custom date field
                    'value' => current_time('mysql'), //current date and time
                    'compare' => '<', //posts with a date older than now
                    'type' => 'DATETIME'
                )
            )
        );
        $job_args = array_merge(
            $this->args,
            $args
        );
        $posts = get_posts($job_args);
        return count($posts);
    }
    /**
     * Deletes all job posts that have passed their deadline.
     *
     * @return void
     */
    public function delete_all_jobs()
    {
        $args = array(
            'fields' => 'ids',    // This line was added
            'meta_query' => array(
                array(
                    'key' => 'job_ad_deadline', //replace with your custom date field
                    'value' => current_time('mysql'), //current date and time
                    'compare' => '<', //posts with a date older than now
                    'type' => 'DATETIME'
                )
            )
        );
        $job_args = array_merge(
            $this->args,
            $args
        );
        $posts = get_posts($job_args);
        $trash = [
            'ids' => $posts
        ];
        foreach ($posts as $post_id) {
            wp_trash_post($post_id);
        }
        wp_send_json_success($trash);
        wp_die();
    }
    /**
     * Builds the arguments for a job.
     *
     * @param array $args The arguments to be merged with the job's default arguments.
     * @return array The final arguments for the job.
     */
    public function build_args($args)
    {

        $job_args = array_merge(
            $this->args,
            $args
        );


        // Handle taxonomies
        $taxonomies = ['jobcategory', 'job_form_of_employment', 'job_working_time', 'working_time', 'job_type'];


        foreach ($taxonomies as $taxonomy) {
            $taxonomy_str = $taxonomy . '_str';

            if (!empty($args[$taxonomy_str])) {

                $job_args['tax_query'][] = [
                    'taxonomy' => $taxonomy,
                    'field' => 'slug',
                    'terms' => strtolower($args[$taxonomy_str]),

                ];
            }


        }

        if (isset($job_args['meta_query'])) {
            $job_args['meta_query'] = array_merge(
                $job_args['meta_query'],
                $this->meta_query
            );
        } else {
            $job_args['meta_query'] = [$this->meta_query];
        }
    

        return $job_args;
    }
    /**
     * Accepts all jobs and changes their status to 'publish'.
     *
     * @param array $args Optional. Array of arguments for retrieving posts.
     * @return void
     */
    public function accept_all_jobs($args = [])
    {
        $args = [
            'post_type' => 'job',
            'posts_per_page' => -1,
            'post_status' => 'pre-pending',
            'fields' => 'ids',    // This line was added
        ];
        $posts = get_posts($args);
        foreach ($posts as $post) {
            $temp = [
                'ID' => $post,
                'post_status' => 'publish'
            ];
        }
        wp_send_json_success();
        wp_die();
    }
    /**
     * Deletes all unconfirmed jobs that have passed their deadline.
     *
     * @param array $args Optional. Arguments for customizing the query. Default empty array.
     * @return void
     */
    public function delete_all_unconfirmed_jobs($args = [])
    {
        $args = array(
            'fields' => 'ids',    // This line was added
            'meta_query' => array(
                array(
                    'key' => 'job_ad_deadline', //replace with your custom date field
                    'value' => current_time('mysql'), //current date and time
                    'compare' => '<', //posts with a date older than now
                    'type' => 'DATETIME'
                )
            )
        );



        $args = [
            'fields' => 'ids',
            'post_type' => 'job',
            'posts_per_page' => -1,
            'post_status' => 'unconfirmed',
        ];
        $posts = get_posts($args);
        foreach ($posts as $post_id) {
            wp_trash_post($post_id);
        }
        wp_send_json_success();
        wp_die();
    }
    /**
     * Retrieves a WP_Query object based on the given arguments.
     *
     * @param array $args Optional. Additional arguments to be merged with the default arguments.
     * @return WP_Query The WP_Query object containing the queried posts.
     */
    public function get_query($args)
    {
        $job_args = array_merge(
            $this->args,
            $args
        );
        // Add 'paged' parameter to query
        $paged = (get_query_var('paged')) ? get_query_var('paged') : 1;
        $job_args['paged'] = $paged;

        // Query for posts
        $query = new WP_Query($job_args);
        return $query;
    }
    /**
     * Retrieves all jobs based on the given arguments.
     *
     * @param array $args Optional. Additional arguments to be merged with the default arguments.
     * @return array An array of job objects.
     */
    public function get_all_jobs($args = [])
    {


        $job_args = array_merge(
            $this->args,
            $args
        );

        if (isset($job_args['meta_query'])) {
            $job_args['meta_query'] = array_merge(
                $job_args['meta_query'],
                $this->meta_query
            );
        } else {
            $job_args['meta_query'] = [$this->meta_query];
        }
        $jobs = [];
      
        $posts = get_posts($job_args);
 
        foreach ($posts as $post) {
            $LGenPost = new LGenPost();
            $job = $LGenPost->setup_job($post);

            $jobs[] = $job;
        }

 

        return $jobs;
    }
    /**
     * Returns the number of posts found by the WP_Query object with the given arguments.
     *
     * @param array $args Optional. Additional arguments to be merged with the default arguments.
     * @return int The number of posts found by the WP_Query object.
     */
    public function found_posts($args = [])
    {

        $job_args = array_merge(
            $this->args,
            $args
        );
        if (isset($job_args['meta_query'])) {
            $job_args['meta_query'] = array_merge(
                $job_args['meta_query'],
                $this->meta_query
            );
        } else {
            $job_args['meta_query'] = [$this->meta_query];
        }

        $query = new \WP_Query($job_args);
        return $query->found_posts;
    }
    /**
     * Queries the wishlist jobs using the given arguments.
     *
     * @param array $args Optional. Additional arguments to be passed to the query.
     * @return string The rendered HTML of the job list.
     */
    public function query_whislist_jobs($args = [])
    {
        $template = apply_filters('wp_leadgen_template_path', 'job-item-list.twig');
        $job_args = array_merge(
            $this->args,
            $args
        );
        $jobs = [];
        $posts = get_posts($job_args);
        foreach ($posts as $post) {
            $LGenPost = new LGenPost();
            $job = $LGenPost->setup_job($post);


            $jobs[] = $job;
        }
        $data = [
            'jobs' => $jobs
        ];

        $rendered_html = Timber::compile($template, $data);
        return $rendered_html;
    }

    /**
     * Retrieves all premium jobs from the database.
     *
     * @param array $args Optional. Additional arguments for querying the database. Default empty.
     * @return array Array of premium job objects.
     */
    public function get_all_premium_jobs($args = [])
    {

        //  var_dump($args);
        $args['posts_per_page'] = 1;


        $meta_query = array(
            array(
                'key' => 'premium',
                'value' => '1',
                'compare' => '=',
            ),
        );


        $job_args = array_merge(
            $this->args,
            $args
        );


        if (isset($job_args['meta_query'])) {
            $job_args['meta_query'] = array_merge(
                $job_args['meta_query'],
                $meta_query
            );
        } else {
            $job_args['meta_query'] = [$meta_query];
        }




        $posts = get_posts($job_args);


        $jobs = [];
        foreach ($posts as $post) {
            $LGenPost = new LGenPost();
            $job = $LGenPost->setup_job($post);

            $jobs[] = $job;
        }

        return $jobs;
    }
    /**
     * Refreshes the jobs by retrieving the post IDs and using them to get the order ID and product refresh value.
     * 
     * @return void
     */
    public function refresh_jobs()
    {
        $args = array(
            'fields' => 'ids',    // This line was added
        );
        $job_args = array_merge(
            $this->args,
            $args
        );
        $posts = get_posts($job_args);

        foreach ($posts as $post_id) {
            $order_id = $this->get_order_id($post_id);
            if ($order_id) {
                $this->update_order_product_refresh_value($order_id, $post_id);
            }
        }
    }
    /**
     * Retrieves the order ID associated with a given job ID.
     *
     * @param int $job_id The ID of the job to retrieve the order ID for.
     * @return int|bool The order ID if found, or false if not found.
     */
    function get_order_id($job_id)
    {
        $order_id = get_field('order_id', $job_id);
        return $order_id;
    }
        /**
     * Retrieves the product ID associated with a given order ID.
     *
     * @param int $order_id The ID of the job to retrieve the order ID for.
     * @return int|bool The order ID if found, or false if not found.
     */
    function get_product_id($order_id)
    {
        $order = new WC_Order($order_id);

        // get the list of all products in this order
        $items = $order->get_items();

        // the first product in this order
        $first_item = reset($items);

        // get first product id
        $product_id = $first_item['product_id'];
        return $product_id;
    }
   /**
     * Retrieves the refresh value for a product in an order based on the job start date.
     *
     * @param int $order_id The ID of the order.
     * @param int $job_id The ID of the job.
     * @return DateInterval|null The refresh period for the product, or null if no refresh period is set.
     */
    function get_order_product_delete_value($order_id, $job_id)
    {

        $order = new WC_Order($order_id);

        // get the list of all products in this order
        $items = $order->get_items();

        // the first product in this order
        $first_item = reset($items);

        // get first product id
        $product_id = $first_item['product_id'];
        $refresh_period = get_post_meta($product_id, 'refresh_period', true);
        $job_start_date = get_post_meta($job_id, 'job_start_date', true);
    
        if (!$job_start_date) {
            $job_start_date = get_the_date('d.m.Y', $job_id);
            update_post_meta($job_id, 'job_start_date', $job_start_date);

            $start_date = DateTime::createFromFormat('Ymd', $job_start_date);
        } else {
            $start_date = DateTime::createFromFormat('Ymd', $job_start_date);
        }



        $now = new DateTime(); // Get the current date
        if ($start_date) {
            $diff = $now->diff($start_date); // Calculate the difference



            $refresh_period = new DateInterval($this->convertToIntervalFormat($refresh_period));

            if ($diff <= $refresh_period) {
                $job_start_date = $start_date->add($refresh_period);

                // Then update the new job start date field
                update_post_meta($job_id, 'job_start_date', $job_start_date->format('Ymd'));
            }
        }
    }

    /**
     * Retrieves the refresh value for a product in an order based on the job start date.
     *
     * @param int $order_id The ID of the order.
     * @param int $job_id The ID of the job.
     * @return DateInterval|null The refresh period for the product, or null if no refresh period is set.
     */
    function update_order_product_refresh_value($order_id, $job_id)
    {
        $order = new WC_Order($order_id);
    
        // Get the list of all products in this order
        $items = $order->get_items();
    
        // Get the first product in this order
        $first_item = reset($items);
    
 

        $new_job_start_date = get_post_meta($job_id, 'job_refresh_date', true);
    
        $start_date = DateTime::createFromFormat('Ymd', $new_job_start_date);
    
        $now = new DateTime(); // Get the current date

        if ($new_job_start_date) {
            $diff = $now->diff($start_date); // Calculate the difference

   
       
            if ($diff->days <= 1) {
        

                // Then update the new job start date field
                update_post_meta($job_id, 'job_start_date',  $new_job_start_date);
            }
        }
   
    }
    /**
     * Converts a given input into an interval format.
     *
     * @param string $input The input to be converted.
     * @return string The converted input in 'PxD' format where x is the numeric period.
     */
    function convertToIntervalFormat($input)
    {
        // Strips non-numeric characters, leaving only the period in numeric form
        $periodNum = preg_replace('/\D/', '', $input);

        // Returns in 'PxD' format where x is the numeric period
        return 'P' . $periodNum . 'D';
    }
}
