<?php
/**
 * Class Shortcodes
 * This class handles shortcodes which allow to display job-related content on WordPress pages.
 *
 * Shortcodes include jobs, wishlist_heart, job_terms_list, job_contact_box, wishlist_reset, and count_jobs.
 *
 * @package LeadGenerator
 * @since 0.2.1
 */
defined('ABSPATH') || exit;
class Shortcodes
{
    /**
     * Constructor for the class.
     * Adds shortcodes for various job-related functions.
     *
     * @since 1.0.0
     */

    public $ARGS;

    public const SHORTCODES = [
        'filter_jobs',
        'filter_sort',
        'filter_count',
        'filter_selected',
        'search_form',
        'filter_pagination',
        'jobs',
        'count_jobs',
        'filter_barbox',
    ];


    function __construct()
    {
        /* Shortcode to display job listings, accepts a range of criteria to select jobs */
        add_shortcode('jobs', [$this, 'query_jobs']);

        /* Shortcode to display a heart-shaped wishlist button with job ID attached */
        add_shortcode('wishlist_heart', [$this, 'show_heart']);

        /* Shortcode to display a list of job-related terms */
        add_shortcode('job_terms_list', [$this, 'query_custom_terms']);

        /* Shortcode to display the contact box for a job post */
        add_shortcode('job_contact_box', [$this, 'job_contact_box']);

        /* Shortcode to reset the wishlist */
        add_shortcode('wishlist_reset', [$this, 'wishlist_reset']);

        /* Shortcode to count and show the number of jobs meeting specified conditions */
        add_shortcode('count_jobs', [$this, 'count_jobs']);

        add_shortcode('filter_barbox', [$this, 'filter_barbox']);

        /* Shortcode to show  conditional content */
        add_shortcode('conditional', [$this, 'conditional_shortcode']);

        $this->ARGS =
            [
                'post_type' => 'job',
                'post_status' => 'publish',
                'per_page' => '10',
                'order' => 'asc',
                'posts_per_page' => -1,
                'orderby' => 'date',
                'jobcategory' => '',
                'job_type' => '',
                'radius' => 25,
                'city' => false,
                'zip' => false,
                'job_form_of_employment_str' => false,
                'job_working_time_str' => false,
                'job_type_str' => false,
                'premium' => false,
                'company_id' => false,
                'post__not_in' => '',
                's' => false,
                'show_pagination' => false,
                'id' => uniqid(),
                'search_placeholder' => false,
                'city_placeholder' => false,

            ];

    }

    /**
     * Renders content based on a specific field's value.
     *
     * Shortcode [conditional]
     *
     * This shortcode allows content to be displayed conditionally based on the
     * value of a user-defined field (via `$attributes['field']`).
     *
     * @since 1.0.0
     *
     * @shortcode [conditional field="" condition="" value=""]
     *
     * @param array $atts {
     *     An array of attributes used by the shortcode.
     *
     *     @type string $field The field name to check its value.
     *     @type boolean $condition The condition to check the field value against. Default 'false'.
     *     @type mixed $value (Optional) value used in the condition. Default 'false'.
     * }
     *
     * @return string Content if condition is met, otherwise an empty string.
     */

    public function conditional_shortcode($attributes, $content = null)
    {
        global $post;
        $attributes = shortcode_atts(
            array(
                'condition' => false,
                'value' => false,
                'field' => false
            ),
            $attributes
        );

        $field_value = get_field($attributes['field'], $post->ID);


        if ($field_value) {
            return do_shortcode($content);
        }

        return '';
    }


    /**
     * Displays the contact box for a job post.
     *
     * @param $post The job post to display the contact box for.
     */
    function job_contact_box()
    {
        global $post;
        LGenPost::show_contact_box($post);


    }

    public static function get_shortcode_global_atts()
    {
        global $post;
        $atts_by_id = [];

        foreach (self::SHORTCODES as $shortcode) {
            if (has_shortcode($post->post_content, $shortcode)) {
                $pattern = get_shortcode_regex();
                if (preg_match_all("/$pattern/", $post->post_content, $matches, PREG_SET_ORDER)) {
                    foreach ($matches as $match) {
                        if ($match[2] === $shortcode) {
                            $atts = shortcode_parse_atts($match[3]);
                            if (isset($atts['id'])) {
                                $atts_by_id[$atts['id']][] = $atts;
                            }

                        }
                    }
                }
            }
        }

        foreach ($atts_by_id as $id => $attributes) {
            $combined_attributes = call_user_func_array('array_merge', $attributes);
            $atts_by_id[$id] = $combined_attributes;
        }


        return $atts_by_id;
    }

    function filter_barbox($atts)
    {

        $args = [
            'term' => 'jobcategory',
            'id' => uniqid(),
            'by' => 'slug',
            'show_count' => false,
            'value' => -1,
        ];
        $args = shortcode_atts(
            $args,
            $atts,

        );

        $Filter = new Filter();

        return $Filter->get_filter_bar($args);
    }
    /**
     * Counts the number of jobs based on the given attributes.
     *
     * @param array $atts The attributes used to filter the jobs.
     * @return string The number of jobs that match the given attributes.
     */
    function count_jobs($atts)
    {
        //  var_dump($atts);
        $args = shortcode_atts(
            [
                'post_type' => 'job',
                'post_status' => 'publish',
                'per_page' => '10',
                'order' => 'asc',
                'posts_per_page' => -1,
                'orderby' => 'date',
                'jobcategory' => '',
                'job_type' => '',
                'radius' => 25,
                'city' => false,
                'zip' => false,
                'job_form_of_employment_str' => false,
                'job_working_time_str' => false,
                'job_type_str' => false,
                'premium' => false,
                'company_id' => false,
                'post__not_in' => '',
                's' => false,
                'show_pagination' => false,
                'id' => uniqid(),
                'search_placeholder' => false,
                'city_placeholder' => false,
            ],
            $atts,

        );


        // Handle distance and location
        if (!empty($args["city"])) {
            if (empty($args["radius"])) {

                $args["radius"] = intval($args["radius"]);

                $place = Nominatim::geocode($args["city"]);
                if ($place) {
                    $args["longitude"] = $place['lon'];
                    $args["latitude"] = $place['lat'];
                }


            } else {
                $args["radius"] = intval($args["radius"]);

                $place = Nominatim::geocode($args["city"]);
                if ($place) {
                    $args["longitude"] = $place['lon'];
                    $args["latitude"] = $place['lat'];
                }


                $posts = Nominatim::get_posts_in_distance($args["longitude"], $args["latitude"], $args["radius"]);

                $IDS = array_map(function ($post) {
                    return $post->ID;
                }, $posts);
                $args['post__in'] = empty($IDS) ? [-1] : $IDS;
            }
        }

        $Filter = new Filter();

        $args = $Filter->build_args($args);

        $jobQuery = new LGenQuery();

        $jobs = $jobQuery->get_all_jobs($args);



        $content = '<span class="count_jobs" id="count_' . $args['id'] . '">' . count($jobs) . '</span>';


        return $content;

    }
    /**
     * Renders a heart icon representing a wishlist button linked to a certain job ID.
     *
     * Shortcode [show_heart]
     *
     * This shortcode is used to create a heart-shaped wishlist button. 
     * The created element is set with an id of the job post to mark it as a favorite.
     * 
     * @since 1.0.0
     *
     * @shortcode [show_heart job_id=""]
     *
     * @param array $atts {
     *     An array of attributes used by the shortcode.
     *
     *     @type int $job_id The job ID to be attached to the wishlist button, defaults to current post's ID.
     * }
     *
     * @return string HTML code for a heart icon (wishlist button) with associated job ID.
     */
    function show_heart($atts)
    {
        global $post;
        $args = shortcode_atts(
            [
                'job_id' => $post->ID
            ],
            $atts,

        );
        return '<div class="icon-text wishlist" data-id="' . $args['job_id'] . '"><i class="far fa-heart"></i><span></span></div>';
    }
    /**
     * Retrieves and display job listings based on specified attributes.
     *
     * Shortcode [jobs]
     * 
     * This shortcode is used to retrieve job posts based on a variety of criteria, 
     * including post type, status, categories, radius, company, and pagination settings. 
     *
     * @since 1.0.0
     *
     * @shortcode [jobs post_type="" post_status="" per_page="" order=""
     *            posts_per_page="" order_by="" jobcategory="" job_type=""
     *            radius="" premium="" company_id="" post__not_in="" s="" show_pagination="" city=""]
     *
     * @param array $atts {
     *     An array of attributes used by the shortcode.
     *
     *     @type string $post_type Type of the post, default 'job'.
     *     @type string $post_status Status of the post, default 'publish'.
     *     @type string $per_page Number of posts to show per page, default '10'.
     *     @type string $order Order in which posts to be displayed, default 'asc'.
     *     @type string $posts_per_page Total number of posts to be displayed, default '-1'.
     *     @type string $order_by Parameter by which the posts should be ordered, default 'date'.
     *     @type string $jobcategory Categories of the job to be query, default ''.
     *     @type string $job_type Types of the job to be query, default ''.
     *     @type string $radius Maximum distance (in miles) for the job location from a given point, default '-1'.
     *     @type string $city City of the job location, default '-1'. If not with radius it will match directly. With radius it will be used as a radius point over gps.
     *     @type string $zip Zip of the job location, default '-1'. If not with radius it will match directly. With radius it will be used as a radius point over gps.
     *     @type string $premium Whether or not to show only premium jobs, default 'false'.
     *     @type string $company_id Company by which the job is posted, default 'false'.
     *     @type string $post__not_in Exclude specific post, default ''.
     *     @type string $s Word to search for in posts, default 'false'.
     *     @type string $show_pagination Whether or not to show pagination, default 'false'.
     * }
     *
     * @return string The HTML output of the job listings.
     */
    function query_jobs($atts)
    {
        $args = shortcode_atts(
            [
                'post_type' => 'job',
                'post_status' => 'publish',
                'per_page' => '10',
                'order' => 'asc',
                'posts_per_page' => -1,
                'orderby' => 'date',
                'jobcategory' => '',
                'job_type' => '',
                'radius' => false,
                'city' => false,
                'zip' => false,
                'job_form_of_employment_str' => false,
                'job_working_time_str' => false,
                'job_type_str' => false,
                'premium' => false,
                'company_id' => false,
                'post__not_in' => '',
                's' => false,
                'search_placeholder' => false,
                'city_placeholder' => false,
                'show_pagination' => false,
                'ID' => uniqid(),
            ],
            $atts,

        );

        $form_id = $args["ID"];


        $output = '<form id="dp-form-' . esc_attr($form_id) . '" class="dp-form-' . esc_attr($form_id) . '">';

        // Also save the args in hidden fields
        foreach ($args as $key => $value) {
            $output .= '<input type="hidden" id="' . $key . '" name="' . $key . '" value="' . $value . '">';
        }
        $output .= '</form>';
        $Filter = new Filter();

        $args = $Filter->build_args($args);


        $output .= '<div id="results-' . esc_attr($form_id) . '">';
        $output .= '<div id="dp-output-' . esc_attr($form_id) . '">';

        $output .= $Filter->get_job_results($args);
        $query = new WP_Query($args);
        $posts_per_page = $query->query_vars['posts_per_page'];

        $paged = $query->query_vars['paged'];
        $found_posts = $query->found_posts;
        $max_num_pages = $query->max_num_pages;
        $output .= '</div>';
        $output .= '</div>';
        if ($posts_per_page <= $found_posts) {
            $output .= $Filter::get_pagination($posts_per_page, $paged, $found_posts, $max_num_pages, $form_id);
        }
        return $output;

    }
    /**
     * Retrieves and displays a list of custom taxonomy terms based on specified attributes.
     *
     * Shortcode [query_custom_terms]
     *
     * This shortcode is used to display a list of custom taxonomy terms. It can be configured
     * by various parameters like taxonomy, term count, sort order, and visibility of empty terms. 
     *
     * @since 1.0.0
     *
     * @shortcode [query_custom_terms taxonomy="" count="" orderby="" order="" hide_empty="" post_types=""]
     *
     * @param array $atts {
     *     An array of attributes used by the shortcode.
     *
     *     @type string $taxonomy The taxonomy for which terms should be retrieved, default 'jobcategory'.
     *     @type bool   $count Whether or not to display the term count, default 'False'.
     *     @type string $orderby The parameter to order terms by, default 'name'.
     *     @type string $order The order to display the terms in, default 'asc'.
     *     @type bool   $hide_empty Whether or not to hide empty terms, default 'True'.
     *     @type string $post_types The post type for which terms are to be retrieved, default 'job'.
     * }
     *
     * @return string HTML output of custom taxonomy terms.
     */
    function query_custom_terms($atts)
    {
        $args = shortcode_atts(
            [
                'taxonomy' => 'jobcategory',
                'count' => False,
                'orderby' => 'name',
                'order' => 'asc',
                'hide_empty' => True,
                'post_types' => 'job',
            ],
            $atts
        );

        // Get the taxonomy terms
        $terms = get_terms(
            $args
        );

        if (empty($terms) || is_wp_error($terms)) {
            return ''; // Handle the error or empty case
        }

        // Create a list of links to the terms
        $output = '<ul class="uk-list uk-text-center">';
        foreach ($terms as $term) {
            $link = get_term_link($term);
            if (is_wp_error($link)) {
                continue; // Handle the error case
            }
            $count = get_term_count($term->term_id, 'jobcategory', 'job');
            if ($count > 0) {

                $output.= "<li class='el-item'><a class='uk-link-toggle' href='" . esc_url($link) . "'>" . esc_html($term->name) . "</a>";

                /*
                $output .= "<li class='el-item'>
                                <a class='uk-link-toggle' href='" . esc_url($link) . "'>
                                    <div class='uk-grid-small uk-child-width-expand uk-flex-nowrap uk-flex-middle uk-grid' uk-grid=''>            
                                        <div class='uk-width-auto uk-link uk-first-column'>
                                            <span class='el-image uk-text-secondary uk-icon' uk-icon='icon: arrow-right;'></span>
                                        </div>            
                                        <div>
                                            <div class='el-content uk-panel'>
                                                <span class='uk-link uk-margin-remove-last-child'>" . esc_html($term->name);
                /*
                // Add count to the output if args['count'] is True
                if ($args['count'] == True) {
                    $output .= " (" . get_term_count($term->term_id, 'jobcategory', 'job');
                }

                $output .= ")
                
                    </span>
                </div>            
            </div>
            </div>            
            </a></li>";
            */
    
        }
        }
        $output .= '</ul>';

        return $output;
    }
}
new Shortcodes();