<?php
/**
 * Lightweb Media License
 * @package lightweb-media/lwm-license
 * @version 1.0.6
 * @link https://lightweb-media.de
 * @since 1.0.0
 * 
 */
namespace Lightweb_Media\Lightweb_Media_License;

defined('ABSPATH') || exit;

if (!function_exists('get_plugin_data')) {
    require_once(ABSPATH . 'wp-admin/includes/plugin.php');
}

class LWM_License
{
    const LWM_LICENSE_API_PATH = 'api';

    const LWM_LICENSE_ACTION_PARAM = 'action';
    const LWM_LICENSE_META_PARAM = 'get_metadata';
    const LWM_LICENSE_SLUG_PARAM = 'slug';
    const LWM_LICENSE_DOWNLOAD_PARAM = 'download';
    const LWM_LICENSE_PARAM = 'license_key';

    public function __construct(
        private string $full_plugin_path = '',
        private string $plugin_key = '',
        private string $plugin_slug = '',
        private string $license_key = '',
        private string $metadata_url = '',
        private string $download_url = '',
        private array $plugin_data = [],
        private string $plugin_version = '',
        private string $cache_key = '',
        private bool $cache_allowed = true,
        private int $cache_expired = 30,
    ) {
        $this->full_plugin_path = $full_plugin_path;
        $this->plugin_key = $plugin_key;
        $this->plugin_slug = $plugin_slug;
        $this->license_key = $license_key;

        // phpcs:ignore
        $this->plugin_data = \get_plugin_data($this->full_plugin_path);

        $this->plugin_version = $this->plugin_data['Version'] ?? '';

        $base_url = add_query_arg(
            [
                self::LWM_LICENSE_SLUG_PARAM => $this->plugin_key

            ],
            $this->plugin_data['UpdateURI'] . '/' . self::LWM_LICENSE_API_PATH
        );

        $this->metadata_url = add_query_arg(
            [
                self::LWM_LICENSE_ACTION_PARAM => self::LWM_LICENSE_META_PARAM,
            ],
            $base_url
        );

        $this->download_url = add_query_arg(
            [
                self::LWM_LICENSE_ACTION_PARAM => self::LWM_LICENSE_DOWNLOAD_PARAM,
                self::LWM_LICENSE_PARAM => $this->license_key
            ],
            $base_url
        );



        $this->cache_key = $this->plugin_key . '_updater';

        // phpcs:disable
        add_filter('plugins_api', array($this, 'info'), 20, 3);
        add_filter('pre_set_site_transient_update_plugins', array($this, 'update'));
        add_action('upgrader_process_complete', array($this, 'purge'), 10, 2);
        add_action('in_plugin_update_message-' . $this->plugin_slug, [$this, 'update_message'], 10, 2);
        // phpcs:enable
    }

    public function update_message($plugin_info_array, $plugin_info_object): false|string
    {
        if (empty($plugin_info_array['license_valid'])) {
            return 'Please renew your license to update. You can change your license key in Settings > General';
        }
    }

    public function update($transient): mixed
    {
        if (empty($transient->checked)) {
            return $transient;
        }

        $remote = $this->request();

        if (!empty($remote) && !empty($this->plugin_version) && !empty($remote->version) && version_compare($this->plugin_version, $remote->version, '<')) {
            $res = $this->prepare_update_response($remote);
            $transient->response[$res->plugin] = $res;
        }

        return $transient;
    }

    private function request(): object
    {
        // phpcs:ignore
        $remote = get_transient($this->cache_key);

        if (empty($remote)) {
            // phpcs:ignore
            $remote = wp_remote_get(
                $this->metadata_url,
                ['timeout' => 10, 'headers' => ['Accept' => 'application/json']]
            );




            // phpcs:ignore
            if (
                is_wp_error($remote) || 200 !== wp_remote_retrieve_response_code($remote) ||
                // phpcs:ignore
                empty(wp_remote_retrieve_body($remote))
            ) {
                return new \stdClass();
            }
            // phpcs:ignore
            set_transient($this->cache_key, $remote, $this->cache_expired);
        }
        // phpcs:ignore
        $plugin_info = json_decode(wp_remote_retrieve_body($remote));
        // phpcs:ignore

        return $plugin_info;
    }

    private function prepare_update_response(object $remote): object
    {

        $res = new \stdClass();
        $res->slug = $this->plugin_key;
        $res->plugin = $this->plugin_slug;
        $res->new_version = $remote->version;
        $res->package = $this->download_url;
        $res->url = $res->url ?? (isset($remote->homepage) ? $remote->homepage : '');
        $res->tested = $res->tested ?? (isset($remote->tested) ? $remote->tested : '');
        $res->requires = $res->requires ?? (isset($remote->requires) ? $remote->requires : '');
        $res->requires_php = $remote->requires_php;
        $res->last_updated = $remote->last_updated;
        if (isset($remote->sections)) {
            $res->sections = [
                'description' => $remote->sections->description,
                'installation' =>
                    $remote->sections->installation,
                'changelog' => $remote->sections->changelog
            ];
        } else {
            $res->sections = [];
        }


        return $res;
    }

    public function purge($upgrader_object, $options): void
    {
        if ($this->cache_allowed && 'update' === $options['action'] && 'plugin' === $options['type']) {
            // phpcs:ignore
            delete_transient($this->cache_key);
        }
    }

    public function info($res, $action, $args): mixed
    {
        if ('plugin_information' !== $action || $this->plugin_key !== $args->slug) {
            return $res;
        }

        $remote = $this->request();

        if (empty($remote)) {
            return $res;
        }
        var_dump($this->download_url);
        $res = new \stdClass();
        $res->name = $remote->name;
        $res->slug = $remote->slug;
        $res->version = $remote->version;
        $res->author = $remote->author;
        $res->author_profile = $remote->author_homepage;
        $res->download_link = $this->download_url;
        $res->trunk = $this->download_url;
        $res->tested = $res->tested ?? (isset($remote->tested) ? $remote->tested : '');
        $res->requires = $res->requires ?? (isset($remote->requires) ? $remote->requires : '');
        $res->requires_php = $remote->requires_php;
        $res->last_updated = $remote->last_updated;
        if (isset($remote->sections)) {
            $res->sections = [
                'description' => $remote->sections->description,
                'installation' =>
                    $remote->sections->installation,
                'changelog' => $remote->sections->changelog
            ];
        } else {
            $res->sections = [];
        }
        if (!empty($remote->banners)) {
            $res->banners = ['low' => $remote->banners->low, 'high' => $remote->banners->high];
        }


        return $res;
    }
}